@@ -61,35 +61,18 @@ clear_all()
 }
 
 /*
- * Allocate the equation spaces.
+ * Allocate the needed temp storage.
  *
  * Return true if successful.
  */
 int
 init_mem()
 {
-	int	i;
-
 	if ((scratch = (token_type *) malloc((n_tokens + 10) * sizeof(token_type))) == NULL
 	    || (tlhs = (token_type *) malloc(n_tokens * sizeof(token_type))) == NULL
 	    || (trhs = (token_type *) malloc(n_tokens * sizeof(token_type))) == NULL) {
 		return false;
 	}
-	for (i = 0; i < N_EQUATIONS; i++) {
-		lhs[i] = (token_type *) malloc(n_tokens * sizeof(token_type));
-		if (lhs[i] == NULL)
-			break;
-		rhs[i] = (token_type *) malloc(n_tokens * sizeof(token_type));
-		if (rhs[i] == NULL) {
-			free(lhs[i]);
-			break;
-		}
-	}
-	n_equations = i;
-	if (n_equations < N_EQUATIONS && n_equations < 5) {
-		return false;
-	}
-	clear_all();
 	return true;
 }
 
@@ -206,7 +189,8 @@ int	have;	/* equation number to solve */
  * to the LHS (via transposition), then moving everything not containing the variable to the
  * RHS.  Many tricks are used, and this routine works very well.
  *
- * Returns true if successful, with the result placed in the passed LHS and RHS.
+ * Returns 1 or 2 if successful, with the result placed in the passed LHS and RHS.
+ * Returns 2 if a solution was zero and removed.
  */
 int
 solve_sub(wantp, wantn, leftp, leftnp, rightp, rightnp)
@@ -222,9 +206,7 @@ int		*rightnp;	/* size of RHS */
 	int		worked;
 	int		diff_sign;
 	int		op, op_kind;
-	token_type	*p1, *b1;
-	token_type	*ep;
-	token_type	want0;
+	token_type	*p1, *b1, *ep, want0;
 	long		v;
 	int		need_flip;
 	int		uf_flag;
@@ -234,7 +216,9 @@ int		*rightnp;	/* size of RHS */
 	int		inc_count;
 	int		zero_solved;
 	double		numerator, denominator;
+	int		success;
 
+	success = 1;
 	repeat_count = 0;
 	qtries = 0;
 	zero_solved = false;
@@ -421,21 +405,17 @@ zero_simp:
 					break;
 				}
 			}
-#if	!SILENT
-			if (debug_level > 0) {
-				if (zsolve) {
-					printf(_("Solve for zero completed:\n"));
-				} else {
-					printf(_("Solve completed:\n"));
-				}
+			if (zsolve) {
+				debug_string(1, _("Solve for zero completed:"));
+			} else {
+				debug_string(1, _("Solve completed:"));
 			}
-#endif
 			list_tdebug(1);
 			blt(leftp, tlhs, n_tlhs * sizeof(*leftp));
 			*leftnp = n_tlhs;
 			blt(rightp, trhs, n_trhs * sizeof(*rightp));
 			*rightnp = n_trhs;
-			return true;
+			return success;
 		}
 	}
 	/* Move what we don't want in the LHS to the RHS. */
@@ -589,6 +569,7 @@ zero_simp:
 				}
 				if (uf_flag) {
 					simps_side(tlhs, &n_tlhs, zsolve);
+					list_tdebug(1);
 					uf_flag = false;
 					goto see_work;
 				}
@@ -610,7 +591,7 @@ zero_simp:
 							    || (tlhs[i+1].kind == VARIABLE && tlhs[i+1].token.variable == v
 							    && (tlhs[i+1].level == 1
 							    || (tlhs[i+1].level == 2 && tlhs[i+2].token.operatr == POWER
-							    && tlhs[i+3].level == 2 && tlhs[i+3].kind == CONSTANT)))) {
+							    && tlhs[i+3].level == 2 && tlhs[i+3].kind == CONSTANT && tlhs[i+3].token.constant > 0.0)))) {
 								op = op_kind;
 								b1 = &tlhs[i+1];
 								if (op_kind == DIVIDE)
@@ -633,12 +614,13 @@ zero_simp:
 					}
 				}
 				if ((zflag && zero_solved && op == TIMES
-				    && b1->kind == VARIABLE && b1->token.variable == v
-				    && (b1->level == 1 || (b1->level == 2 && (b1 + 1)->token.operatr == POWER
-				    && (b1 + 2)->level == 2 && (b1 + 2)->kind == CONSTANT)))
+				    && b1[0].kind == VARIABLE && b1[0].token.variable == v
+				    && (b1[0].level == 1 || (b1[0].level == 2 && b1[1].token.operatr == POWER
+				    && b1[2].level == 2 && b1[2].kind == CONSTANT && b1[2].token.constant > 0.0)))
 				    || op == DIVIDE) {
 					if (op == TIMES) {
 						qtries = 0;	/* might be quadratic after removing solution */
+						success = 2;
 #if	!SILENT
 						printf(_("Removing possible solution: \""));
 						list_proc(b1, 1, false);
@@ -726,8 +708,9 @@ fin1:
 }
 
 /*
- * Isolate the expression containing "v" raised to the power of "d",
- * then raise both sides of the equation to the power of 1/"d".
+ * Isolate the expression containing variable "v" raised to the power of "d",
+ * then raise both sides of the equation to the power of 1/d.
+ *
  * Return true if successful.
  */
 static int
@@ -827,11 +810,7 @@ lonely:
 		    && b1->kind == CONSTANT && b1->token.constant == 1.0) {
 			if (!flip(tlhs, &n_tlhs, trhs, &n_trhs))
 				return false;
-			list_tdebug(2);
-			simp_loop(tlhs, &n_tlhs);
-			simp_loop(trhs, &n_trhs);
-			list_tdebug(1);
-			goto lonely;
+			goto end;
 		}
 		switch (p1->token.operatr) {
 		case TIMES:
@@ -849,6 +828,7 @@ lonely:
 	}
 	if (!g_of_f(op, b1, tlhs, &n_tlhs, trhs, &n_trhs))
 		return false;
+end:
 	list_tdebug(2);
 	simp_loop(tlhs, &n_tlhs);
 	simp_loop(trhs, &n_trhs);
@@ -861,14 +841,15 @@ lonely:
  * Solves any equation of the form "0 = ax^(2n)+bx^n+c" for "x",
  * where "x" is an expression containing passed variable "v",
  * and "n" is a constant.
+ *
+ * Return true if successful.
  */
 static int
 quad_solve(v)
 long	v;
 {
 	int		i, j, k;
-	token_type	*p1, *ep;
-	token_type	*p2;
+	token_type	*p1, *p2, *ep;
 	token_type	*x1p, *x2p;
 	token_type	*a1p, *a2p, *a2ep;
 	token_type	*b1p, *b2p, *b2ep;
@@ -1209,10 +1190,10 @@ big_bbreak:
 static int
 g_of_f(op, operandp, side1p, side1np, side2p, side2np)
 int		op;
-token_type	*operandp;
-token_type	*side1p;
+token_type	*operandp;	/* operand pointer */
+token_type	*side1p;	/* equation side pointer */
 int		*side1np;	/* pointer to the length of "side1p" */
-token_type	*side2p;
+token_type	*side2p;	/* equation side pointer */
 int		*side2np;	/* pointer to the length of "side2p" */
 {
 	int		i;
@@ -1455,17 +1436,19 @@ int		*side2np;	/* pointer to the length of "side2p" */
 
 /*
  * Take the reciprocal of both equation sides.
+ *
+ * Return true if successful.
  */
 static int
 flip(side1p, side1np, side2p, side2np)
-token_type	*side1p;
-int		*side1np;
+token_type	*side1p;	/* equation side pointer */
+int		*side1np;	/* pointer to equation side length */
 token_type	*side2p;
 int		*side2np;
 {
 	token_type	*p1, *ep;
 
-	debug_string(0, _("Taking the reciprocal of both sides of the equation..."));
+	debug_string(1, _("Taking the reciprocal of both sides of the equation..."));
 	if (*side1np + 2 > n_tokens || *side2np + 2 > n_tokens) {
 		error_huge();
 	}
@@ -1479,6 +1462,7 @@ int		*side2np;
 	*side1np += 2;
 	blt(side2p + 2, side2p, *side2np * sizeof(*side2p));
 	*side2np += 2;
+
 	side1p->level = 1;
 	side1p->kind = CONSTANT;
 	side1p->token.constant = 1.0;
@@ -1503,15 +1487,30 @@ int		*side2np;
 int
 next_espace()
 {
-	int	i,n;
+	int	i, n;
 
 	for (n = cur_equation, i = 1;; n = (n + 1) % n_equations, i++) {
 		if (i > n_equations) {
+			i = n_equations;
+			if (i < N_EQUATIONS) {
+				lhs[i] = (token_type *) malloc(n_tokens * sizeof(token_type));
+				if (lhs[i]) {
+					rhs[i] = (token_type *) malloc(n_tokens * sizeof(token_type));
+					if (rhs[i] == NULL) {
+						free(lhs[i]);
+						lhs[i] = NULL;
+					}
+				}
+			}
+			if (i >= N_EQUATIONS || lhs[i] == NULL) {
 #if	!SILENT
-			printf(_("Out of free equation spaces.\n"));
-			printf(_("Please use the \"clear\" command on unnecessary equations and try again.\n"));
+				printf(_("Out of free equation spaces.\n"));
+				printf(_("Please use the \"clear\" command on unnecessary equations and try again.\n"));
 #endif
-			longjmp(jmp_save, 3);
+				longjmp(jmp_save, 3);
+			}
+			n_equations++;
+			return i;
 		}
 		if (n_lhs[n] == 0)
 			break;
@@ -1525,8 +1524,8 @@ next_espace()
  */
 int
 min_level(equation, n)
-token_type	*equation;
-int		n;
+token_type	*equation;	/* expression pointer */
+int		n;		/* expression size */
 {
 	int		min1;
 	token_type	*p1, *ep;
