@@ -49,12 +49,12 @@
  * The following defines the maximum number of equation spaces
  * that will be allocated.
  */
-#define	N_EQUATIONS	40
+#define	N_EQUATIONS	100
 
 /*
  * The following defines the default maximum mathematical expression size.
  */
-#define	DEFAULT_N_TOKENS	7500
+#define	DEFAULT_N_TOKENS	15000
 
 #define	DIVISOR_SIZE	(DEFAULT_N_TOKENS / 2)	/* A nice maximum divisor size. */
 
@@ -72,17 +72,6 @@
 
 typedef	char	sign_array_type[MAX_SUBSCRIPT+2];
 
-/*
- * The following structure is used for each Mathomatic command.
- */
-typedef	struct {
-	char	*name;		/* command name to be typed by user */
-	int	(*func)();	/* function that handles this command */
-				/* function is passed a char pointer and returns true if successful */
-	char	*usage;		/* command syntax */
-	char	*info;		/* short description of command */
-} com_type;
-
 enum kind_list {		/* kinds of tokens specified in the union below */
 	CONSTANT,
 	VARIABLE,
@@ -134,11 +123,13 @@ typedef struct {		/* variable sort data structure */
 } sort_type;
 
 #if	SILENT
+#define	error(str)
 #define	list_tdebug(level)
 #define	side_debug(level, p1, n1)
 #define debug_string(level, str)
 #else
+#define	error(str)			{ printf("%s\n", str); }
 #define list_tdebug(level)		list_debug(level, tlhs, n_tlhs, trhs, n_trhs)
 #define	side_debug(level, p1, n1)	list_debug(level, p1, n1, NULL, 0)
-#define debug_string(level, str)	{ if (debug_level >= level) printf("%s\n", str); }
+#define debug_string(level, str)	{ if (debug_level >= (level)) printf("%s\n", str); }
 #endif
