@@ -9,7 +9,7 @@
 #include <readline/readline.h>
 #endif
 
-#define	OPT_MIN_SIZE	5	/* Minimum size of repeated expressions to find in "optimize" command. */
+#define	OPT_MIN_SIZE	7	/* Minimum size of repeated expressions to find in "optimize" command. */
 
 static int	sum_product();
 static int	find_more();
@@ -55,6 +55,7 @@ int	product_flag;
 	int		n, ns;
 	token_type	*dest, *source;
 	int		count_down;
+	char		buf[MAX_CMD_LEN];
 
 	v = 0L;
 	if (notdefined(cur_equation)) {
@@ -96,7 +97,7 @@ int	product_flag;
 		}
 	}
 	if (!found_var(source, ns, v)) {
-		printf(_("Variable not found.\n"));
+		error(_("Variable not found."));
 		return false;
 	}
 	if (*cp) {
@@ -104,12 +105,12 @@ int	product_flag;
 	} else {
 		list_var(v, false);
 		snprintf(prompt_str, sizeof(prompt_str), "%s = ", var_str);
-		if ((cp1 = getstring((char *) scratch, n_tokens * sizeof(token_type))) == NULL)
+		if ((cp1 = getstring(buf, sizeof(buf))) == NULL)
 			return false;
 	}
 	d1 = strtod(cp1, &cp);
 	if (cp1 == cp) {
-		printf(_("Invalid number.\n"));
+		error(_("Invalid number."));
 		return false;
 	}
 	cp = skip_space(cp);
@@ -117,12 +118,12 @@ int	product_flag;
 		cp1 = cp;
 	} else {
 		my_strlcpy(prompt_str, _("To: "), sizeof(prompt_str));
-		if ((cp1 = getstring((char *) scratch, n_tokens * sizeof(token_type))) == NULL)
+		if ((cp1 = getstring(buf, sizeof(buf))) == NULL)
 			return false;
 	}
 	d2 = strtod(cp1, &cp);
 	if (cp1 == cp) {
-		printf(_("Invalid number.\n"));
+		error(_("Invalid number."));
 		return false;
 	}
 	if (extra_garbage(cp))
@@ -173,7 +174,7 @@ int	product_flag;
 	} else {
 		n_lhs[result_en] = n;
 	}
-	list_sub(result_en);
+	return_result(result_en);
 	return true;
 }
 
@@ -346,56 +347,48 @@ optimize_cmd(cp)
 char	*cp;
 {
 	int	i, j, k;
+	int	start, stop;
 	int	i1;
-	int	rv;
+	int	rv, flag;
 
-	if ((i = get_default_en(cp)) < 0)
-		return false;
 	rv = false;
-	opt_en[0] = -1;
-	simp_sub(i);
-	for (j = 0; j < n_lhs[i]; j += 2) {
-		if (lhs[i][j].kind == VARIABLE
-		    && (lhs[i][j].token.variable & VAR_MASK) == V_TEMP) {
-			rv = true;
-		}
-	}
-	for (j = 0; j < n_rhs[i]; j += 2) {
-		if (rhs[i][j].kind == VARIABLE
-		    && (rhs[i][j].token.variable & VAR_MASK) == V_TEMP) {
-			rv = true;
-		}
-	}
-	if (rv) {
-		printf(_("Temporary variable \"temp\" already in use in specified equation.\n"));
-		printf(_("Please rename.\n"));
+	if (!get_range_eol(&cp, &start, &stop)) {
 		return false;
 	}
-	while (opt_es(lhs[i], &n_lhs[i])) {
-		rv = true;
-	}
-	while (opt_es(rhs[i], &n_rhs[i])) {
-		rv = true;
-	}
-	if (rv) {
-		for (i1 = 0; opt_en[i1] >= 0; i1++) {
-			for (j = 0; opt_en[j] >= 0; j++) {
-				for (k = j + 1; opt_en[k] >= 0; k++) {
-					while (find_more(rhs[opt_en[k]], &n_rhs[opt_en[k]], opt_en[j]))
-						;
-					while (find_more(rhs[opt_en[j]], &n_rhs[opt_en[j]], opt_en[k]))
-						;
+	for (i = start; i <= stop; i++) {
+		if (n_lhs[i] == 0)
+			continue;
+		opt_en[0] = -1;
+		simp_sub(i);
+		flag = false;
+		while (opt_es(lhs[i], &n_lhs[i])) {
+			flag = true;
+		}
+		while (opt_es(rhs[i], &n_rhs[i])) {
+			flag = true;
+		}
+		if (flag) {
+			rv = true;
+			for (i1 = 0; opt_en[i1] >= 0; i1++) {
+				for (j = 0; opt_en[j] >= 0; j++) {
+					for (k = j + 1; opt_en[k] >= 0; k++) {
+						while (find_more(rhs[opt_en[k]], &n_rhs[opt_en[k]], opt_en[j]))
+							;
+						while (find_more(rhs[opt_en[j]], &n_rhs[opt_en[j]], opt_en[k]))
+							;
+					}
 				}
+				while (opt_es(rhs[opt_en[i1]], &n_rhs[opt_en[i1]]))
+					;
 			}
-			while (opt_es(rhs[opt_en[i1]], &n_rhs[opt_en[i1]]))
-				;
+			for (j = 0; opt_en[j] >= 0; j++) {
+				list_sub(opt_en[j]);
+			}
+			list_sub(i);
 		}
-		for (j = 0; opt_en[j] >= 0; j++) {
-			list_sub(opt_en[j]);
-		}
-		list_sub(i);
-	} else {
-		printf(_("Unable to find any repeated expressions.\n"));
+	}
+	if (!rv) {
+		error(_("Unable to find any repeated expressions."));
 	}
 	return rv;
 }
@@ -412,12 +405,21 @@ char	*cp;
 
 	if ((i = get_default_en(cp)) < 0)
 		return false;
-	high_prec = true;
-	add_history(list_equation(i, false));
-	high_prec = false;
+	push_en(i);
 	debug_string(0, _("Equation pushed.  Hit the UP key to access."));
 	return true;
 }
+
+/*
+ * Push an equation space into the readline history.
+ */
+push_en(en)
+int	en;
+{
+	high_prec = true;
+	add_history(list_equation(en, false));
+	high_prec = false;
+}
 #endif
 
 /*
@@ -511,7 +513,7 @@ char	*cp;
 	if (strncasecmp(cp, "dir", 3) == 0) {
 		cp = skip_param(cp);
 		if (chdir(cp)) {
-			printf(_("Error changing directory.\n"));
+			error(_("Error changing directory."));
 			return false;
 		}
 		return true;
@@ -525,7 +527,7 @@ char	*cp;
 			debug_level = 0;
 		} else {
 			if (*cp == '\0') {
-				printf(_("Please specify a debug level number.\n"));
+				error(_("Please specify a debug level number."));
 				return false;
 			}
 			debug_level = atoi(cp);
@@ -540,7 +542,7 @@ char	*cp;
 			screen_columns = 0;
 		} else {
 			if (*cp == '\0') {
-				printf(_("Please specify how wide the screen is in columns.\n"));
+				error(_("Please specify how wide the screen is in columns."));
 				return false;
 			}
 			screen_columns = atoi(cp);
@@ -583,7 +585,7 @@ char	*cp;
 		finance_option = !negate;
 		return true;
 	}
-	printf(_("Unknown set option \"%s\".\n"), cp);
+	error(_("Unknown set option."));
 	return false;
 }
 
@@ -595,7 +597,7 @@ pause_cmd(cp)
 char	*cp;
 {
 	char	*cp1;
-	char	buf[100];
+	char	buf[MAX_CMD_LEN];
 
 	if (*cp == '\0') {
 		cp = _("Please Press Enter");
@@ -658,7 +660,7 @@ int	imag_flag;
 	if ((i = get_default_en(cp)) < 0)
 		return false;
 	if (n_lhs[i] != 1) {
-		printf(_("Please solve this equation first.\n"));
+		error(_("Please solve this equation first."));
 		return false;
 	}
 	j = next_espace();
@@ -701,18 +703,18 @@ int	imag_flag;
 		}
 	}
 	if (!has_imag) {
-		printf(_("No imaginary parts found.  Equation is not complex.\n"));
+		error(_("No imaginary parts found.  Equation is not complex."));
 		return false;
 	}
 	if (!has_real) {
-		printf(_("No real parts found.  Equation is not complex.\n"));
+		error(_("No real parts found.  Equation is not complex."));
 		return false;
 	}
 	blt(lhs[j], lhs[i], n_lhs[i] * sizeof(token_type));
 	n_lhs[j] = n_lhs[i];
 	simp_divide(rhs[j], &n_rhs[j], 0L);
 	cur_equation = j;
-	list_sub(cur_equation);
+	return_result(cur_equation);
 	return true;
 }
 
@@ -720,7 +722,7 @@ int	imag_flag;
  * The real command.
  */
 int
-real(cp)
+real_cmd(cp)
 char	*cp;
 {
 	return complex_sub(cp, false);
@@ -730,7 +732,7 @@ char	*cp;
  * The imaginary command.
  */
 int
-imaginary(cp)
+imaginary_cmd(cp)
 char	*cp;
 {
 	return complex_sub(cp, true);
@@ -757,7 +759,7 @@ int		*np;
  * The tally command.
  */
 int
-tally(cp)
+tally_cmd(cp)
 char	*cp;
 {
 	int	i;
@@ -834,16 +836,14 @@ char	*cp;
 			return false;
 		}
 		iterations = strtol(cp, &cp, 10);
-		if (iterations <= 0) {
-			printf(_("Invalid number of iterations.\n"));
+		if (*cp || iterations <= 0) {
+			error(_("Invalid number of iterations."));
 			return false;
 		}
-		if (extra_garbage(cp))
-			return false;
 	}
 	if (it_v) {
 		if (!found_var(source, n, it_v)) {
-			printf(_("Iteration variable not found.\n"));
+			error(_("Iteration variable not found."));
 			return false;
 		}
 	}
@@ -994,7 +994,7 @@ int	*diff_signp;
 
 	rv = se_compare(rhs[i], n_rhs[i], rhs[i], n_rhs[i], diff_signp);
 	if (!rv || *diff_signp) {
-		printf(_("Error in compare function or too many terms to compare!\n"));
+		error(_("Error in compare function or too many terms to compare!"));
 		return false;
 	}
 	sign_flag = true;
@@ -1027,10 +1027,12 @@ char	*cp;
 		return false;
 	}
 	if (i == j) {
-		printf(_("Cannot compare an equation with itself.\n"));
+		error(_("Cannot compare an equation with itself."));
 		return false;
 	}
+#if	!SILENT
 	printf(_("Comparing equation #%d with #%d...\n"), i + 1, j + 1);
+#endif
 	already_solved = (solved_equation(i) && solved_equation(j));
 	if (already_solved) {
 		simp_loop(rhs[i], &n_rhs[i]);
@@ -1062,7 +1064,7 @@ char	*cp;
 	want.token.constant = 0.0;
 	if (!solve_sub(&want, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i])
 	    || !solve_sub(&want, 1, lhs[j], &n_lhs[j], rhs[j], &n_rhs[j])) {
-		printf(_("Can't solve for zero!\n"));
+		error(_("Can't solve for zero!"));
 		return false;
 	}
 	uf_simp(rhs[i], &n_rhs[i]);
@@ -1080,7 +1082,7 @@ char	*cp;
 	}
 	if (!solve_sub(&want, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i])
 	    || !solve_sub(&want, 1, lhs[j], &n_lhs[j], rhs[j], &n_rhs[j])) {
-		printf(_("Can't solve for zero!\n"));
+		error(_("Can't solve for zero!"));
 		return false;
 	}
 	uf_simp(rhs[i], &n_rhs[i]);
@@ -1140,15 +1142,21 @@ char	*cp;
 		return false;
 	}
 	printf("\n");
-	simp_loop(lhs[i], &nl);
-	simp_loop(rhs[i], &nr);
-	uf_simp(lhs[i], &nl);
-	uf_simp(rhs[i], &nr);
+	calc_simp(lhs[i], &nl);
+	calc_simp(rhs[i], &nr);
 	if (get_constant(rhs[i], nr, &d1) && get_constant(lhs[i], nl, &d2)) {
 		d4 = modf(d1 / d2, &d3);
 		printf(_("Result of numerical division: %.14g\n"), d1 / d2);
 		printf(_("Quotient: %.14g, Remainder: %.14g\n"), d3, d4 * d2);
-		ngcd(d1, d2);
+		d1 = fabs(d1);
+		d2 = fabs(d2);
+		d3 = gcd(d1, d2);
+		if (d3 <= 0.0) {
+			printf(_("No GCD found.\n"));
+			return true;
+		}
+		printf(_("Greatest Common Divisor (GCD) = %.14g\n"), d3);
+		printf(_("Least Common Multiple (LCM) = %.14g\n"), (d1 * d2) / d3);
 		return true;
 	}
 	if (parse_complex_exp(rhs[i], nr, &c1) && parse_complex_exp(lhs[i], nl, &c2)) {
@@ -1194,7 +1202,7 @@ char	*cp;
  * The eliminate command.
  */
 int
-eliminate(cp)
+eliminate_cmd(cp)
 char	*cp;
 {
 	long	v, last_v, v1;
@@ -1257,7 +1265,7 @@ next_var:
 		v = va[--vc];
 	} else {
 		if (did_something) {
-			list_sub(cur_equation);
+			return_result(cur_equation);
 		}
 		return did_something;
 	}
@@ -1269,13 +1277,12 @@ next_var:
 	using_flag = (strcmp_tospace(cp, "using") == 0);
 	if (using_flag) {
 		cp = skip_space(cp + 5);
-		i = atoi(cp) - 1;
-		cp = skip_num(cp);
+		i = strtol(cp, &cp, 10) - 1;
 		if (notdefined(i)) {
 			return false;
 		}
 		if (i == cur_equation) {
-			printf(_("Error: source and destination are the same.\n"));
+			error(_("Error: source and destination are the same."));
 			return false;
 		}
 		if (!elim_sub(i, v))
@@ -1423,7 +1430,6 @@ flist_cmd(cp)
 char	*cp;
 {
 	int	i, j;
-	jmp_buf	save_save;
 	int	factor_flag;
 
 	if (factor_flag = (strcmp_tospace(cp, "factor") == 0)) {
@@ -1432,11 +1438,6 @@ char	*cp;
 	if (!get_range_eol(&cp, &i, &j)) {
 		return false;
 	}
-	blt(save_save, jmp_save, sizeof(jmp_save));
-	if (setjmp(jmp_save) != 0) {
-		i++;
-		printf(_("Skipping equation #%d.\n"), i);
-	}
 	for (; i <= j; i++) {
 		if (n_lhs[i] > 0) {
 			group_sub(i);
@@ -1447,7 +1448,6 @@ char	*cp;
 			flist_sub(i);
 		}
 	}
-	blt(jmp_save, save_save, sizeof(jmp_save));
 	return true;
 }
 
@@ -1517,10 +1517,7 @@ char	*cp;
 					continue;
 				}
 			}
-			list_c_code(lhs[i], n_lhs[i], java_flag, int_flag);
-			fprintf(gfp, " = ");
-			list_c_code(rhs[i], n_rhs[i], java_flag, int_flag);
-			fprintf(gfp, ";\n\n");
+			list_c_equation(i, java_flag, int_flag);
 		}
 	}
 	return true;
@@ -1530,7 +1527,7 @@ char	*cp;
  * The replace command.
  */
 int
-replace(cp)
+replace_cmd(cp)
 char	*cp;
 {
 	int	i, j;
@@ -1554,7 +1551,7 @@ char	*cp;
 		subst_constants(lhs[i], &n_lhs[i]);
 		subst_constants(rhs[i], &n_rhs[i]);
 		simp_sub(i);
-		list_sub(i);
+		return_result(i);
 		return true;
 	}
 	for (vc = 0; *cp; vc++) {
@@ -1564,7 +1561,7 @@ char	*cp;
 			return false;
 		}
 		if (vc >= ARR_CNT(va)) {
-			printf(_("Too many variables specified!\n"));
+			error(_("Too many variables specified."));
 			return false;
 		}
 		cp = parse_var2(&va[vc], cp);
@@ -1572,8 +1569,7 @@ char	*cp;
 			return false;
 		}
 		if (!var_in_equation(i, va[vc])) {
-			list_var(va[vc], false);
-			printf(_("Variable (%s) not found in current equation.\n"), var_str);
+			error(_("Variable not found."));
 			return false;
 		}
 	}
@@ -1653,7 +1649,7 @@ do_this:
 	n_rhs[i] = n_trhs;
 	blt(rhs[i], trhs, n_trhs * sizeof(token_type));
 	simp_sub(i);
-	list_sub(i);
+	return_result(i);
 	return true;
 }
 
@@ -1661,7 +1657,7 @@ do_this:
  * The simplify command.
  */
 int
-simplify(cp)
+simplify_cmd(cp)
 char	*cp;
 {
 	int	i, j;
@@ -1695,7 +1691,7 @@ char	*cp;
 		if (n_lhs[i]) {
 			simpa_side(lhs[i], &n_lhs[i], poly_flag, quick_flag);
 			simpa_side(rhs[i], &n_rhs[i], poly_flag, quick_flag);
-			list_sub(i);
+			return_result(i);
 		}
 	}
 	symb_flag = false;
@@ -1706,27 +1702,28 @@ char	*cp;
  * The factor command.
  */
 int
-factor(cp)
+factor_cmd(cp)
 char	*cp;
 {
 	int	i, j;
 	int	i1;
 	long	v;
 	double	d;
+	char	buf[MAX_CMD_LEN];
 
 	if (strncasecmp(cp, "number", 6) == 0) {
 		cp = skip_param(cp);
 next_number:
 		if (*cp == '\0') {
 			my_strlcpy(prompt_str, _("Enter integers to factor: "), sizeof(prompt_str));
-			cp = getstring((char *) scratch, n_tokens * sizeof(token_type));
+			cp = getstring(buf, sizeof(buf));
 			if (cp == NULL)
 				return false;
 		}
 		d = strtod(cp, &cp);
 		cp = skip_space(cp);
 		if (!factor_one(d)) {
-			printf(_("Not a valid integer.\n"));
+			error(_("Not a valid integer."));
 			return false;
 		}
 		printf(_("%.0f = "), d);
@@ -1763,7 +1760,7 @@ next_number:
 		}
 	} while (*cp);
 	for (i1 = i; i1 <= j; i1++) {
-		list_sub(i1);
+		return_result(i1);
 	}
 	return true;
 }
@@ -1772,7 +1769,7 @@ next_number:
  * The unfactor command.
  */
 int
-unfactor(cp)
+unfactor_cmd(cp)
 char	*cp;
 {
 	int	i, j;
@@ -1789,7 +1786,7 @@ char	*cp;
 		if (n_lhs[i]) {
 			uf_simp(lhs[i], &n_lhs[i]);
 			uf_simp(rhs[i], &n_rhs[i]);
-			list_sub(i);
+			return_result(i);
 		}
 	}
 	partial_flag = true;
@@ -1821,6 +1818,7 @@ char	*cp;
 	char	buf[MAX_CMD_LEN];
 
 	if (*cp == '\0') {
+		error(_("No file specified."));
 		return false;
 	}
 	fp = NULL;
@@ -1829,7 +1827,7 @@ char	*cp;
 	if (fp == NULL) {
 		fp = fopen(cp, "r");
 		if (fp == NULL) {
-			printf(_("Can't open file \"%s\" or \"%s\".\n"), buf, cp);
+			error(_("Can't open file."));
 			return false;
 		}
 	}
@@ -1837,7 +1835,7 @@ char	*cp;
 	if ((rv = setjmp(jmp_save)) != 0) {
 		clean_up();
 		if (rv == 14)
-			printf(_("Expression too big!\n"));
+			error(_("Expression too big!"));
 		printf(_("Read operation aborted.\n"));
 		goto end_read;
 	}
@@ -1898,7 +1896,7 @@ load_rc()
  * The edit command.
  */
 int
-edit(cp)
+edit_cmd(cp)
 char	*cp;
 {
 	FILE	*fp;
@@ -1911,14 +1909,14 @@ char	*cp;
 		my_strlcpy(tmp_file, "mathxxx.tmp", sizeof(tmp_file));
 		fp = fopen(tmp_file, "w+");
 		if (fp == NULL) {
-			printf(_("Can't create temporary file.\n"));
+			error(_("Can't create temporary file."));
 			return false;
 		}
 #else
 		my_strlcpy(tmp_file, TMP_FILE, sizeof(tmp_file));
 		fd = mkstemp(tmp_file);
 		if (fd < 0 || (fp = fdopen(fd, "w+")) == NULL) {
-			printf(_("Can't create temporary file.\n"));
+			error(_("Can't create temporary file."));
 			return false;
 		}
 #endif
@@ -1933,7 +1931,7 @@ char	*cp;
 		return rv;
 	} else {
 		if (access(cp, 6)) {
-			printf(_("You can only edit existing/writable files or all equations.\n"));
+			error(_("You can only edit existing/writable files or all equations."));
 			return false;
 		}
 		return edit_sub(cp);
@@ -1944,7 +1942,7 @@ static int
 edit_sub(cp)
 char	*cp;
 {
-	char	cl[MAX_CMD_LEN];
+	char	cl[MAX_CMD_LEN];	/* command line */
 	char	*cp1;
 	char	*editor_keyword = "EDITOR";
 
@@ -1981,20 +1979,22 @@ edit_again:
  * The save command.
  */
 int
-save(cp)
+save_cmd(cp)
 char	*cp;
 {
 	FILE	*fp;
 	char	*cp1;
+	char	buf[MAX_CMD_LEN];
 	int	rv;
 
 	if (*cp == '\0') {
+		error(_("No file specified."));
 		return false;
 	}
 	if (access(cp, 0) == 0) {
 		do {
 			snprintf(prompt_str, sizeof(prompt_str), _("\"%s\" exists.  Overwrite (Y/N)? "), cp);
-			if ((cp1 = getstring((char *) trhs, n_tokens * sizeof(token_type))) == NULL) {
+			if ((cp1 = getstring(buf, sizeof(buf))) == NULL) {
 				return false;
 			}
 			str_tolower(cp1);
@@ -2006,7 +2006,7 @@ char	*cp;
 	}
 	fp = fopen(cp, "w");
 	if (fp == NULL) {
-		printf(_("Can't create file \"%s\".\n"), cp);
+		error(_("Can't create file."));
 		return false;
 	}
 	gfp = fp;
@@ -2019,7 +2019,7 @@ char	*cp;
 	if (rv) {
 		printf(_("All equations saved in file: \"%s\".\n"), cp);
 	} else {
-		printf(_("Error encountered while saving equations.\n"));
+		error(_("Error encountered while saving equations."));
 	}
 	return rv;
 }
@@ -2099,7 +2099,7 @@ long	*vp;
 	if (cp1 == NULL)
 		return false;
 	if (*cp1) {
-		printf(_("Only one variable may be specified.\n"));
+		error(_("Only one variable may be specified."));
 		return false;
 	}
 	return true;
@@ -2112,12 +2112,8 @@ int
 notdefined(i)
 int	i;
 {
-	if (i < 0 || i >= n_equations) {
-		printf(_("Invalid equation number.\n"));
-		return true;
-	}
-	if (n_lhs[i] <= 0) {
-		printf(_("Undefined equation.\n"));
+	if (i < 0 || i >= n_equations || n_lhs[i] <= 0) {
+		error(_("Undefined equation."));
 		return true;
 	}
 	return false;
@@ -2170,3 +2166,12 @@ int	n;		/* maximum size of "string" in bytes */
 	}
 	return string;
 }
+
+/*
+ * Store or display the result of a command.
+ */
+return_result(en)
+int	en;
+{
+	list_sub(en);
+}
