@@ -1,5 +1,5 @@
 /*
- * Algebraic manipulator complex number routines.
+ * Floating point complex number library and routines.
  *
  * Copyright (c) 1996 George Gesslein II.
  */
@@ -19,6 +19,32 @@ complexs	*ap;
 	}
 }
 
+#if	false
+/*
+ * Add two complex numbers and return result.
+ */
+complexs
+complex_add(a, b)
+complexs	a, b;
+{
+	a.re += b.re;
+	a.im += b.im;
+	return(a);
+}
+
+/*
+ * Subtract two complex numbers and return result.
+ */
+complexs
+complex_sub(a, b)
+complexs	a, b;
+{
+	a.re -= b.re;
+	a.im -= b.im;
+	return(a);
+}
+#endif
+
 /*
  * Multiply two complex numbers and return result.
  */
@@ -39,8 +65,8 @@ complexs	b;
  */
 complexs
 complex_div(a, b)
-complexs	a;
-complexs	b;
+complexs	a;	/* dividend */
+complexs	b;	/* divisor */
 {
 	complexs	r, num;
 	double		denom;
@@ -112,38 +138,40 @@ int
 roots_cmd(cp)
 char	*cp;
 {
+#define	MAX_ROOT	1000.0
+
 	complexs	c, c2, check;
 	double		d, k;
 	double		root;
 	double		radius, theta;
 	double		radius_root;
-	char		*cp1;
+	char		*cp1, buf[MAX_CMD_LEN];
 
 	if (extra_garbage(cp))
 		return false;
 	for (;;) {
 		my_strlcpy(prompt_str, _("Enter root (positive integer): "), sizeof(prompt_str));
-		if ((cp1 = getstring((char *) scratch, n_tokens * sizeof(token_type))) == NULL)
+		if ((cp1 = getstring(buf, sizeof(buf))) == NULL)
 			return false;
 		if (*cp1 == '\0') {
 			return false;
 		}
 		root = strtod(cp1, &cp1);
-		if (*cp1 || root <= 0.0 || root >= 1000.0 || fmod(root, 1.0) != 0.0) {
-			printf(_("Root must be a positive integer less than 1000.\n"));
+		if (*cp1 || root <= 0.0 || root >= MAX_ROOT || fmod(root, 1.0) != 0.0) {
+			printf(_("Root must be a positive integer less than %.12g\n"), MAX_ROOT);
 			continue;
 		}
 		break;
 	}
 	do {
 		my_strlcpy(prompt_str, _("Enter real part: "), sizeof(prompt_str));
-		if ((cp1 = getstring((char *) scratch, n_tokens * sizeof(token_type))) == NULL)
+		if ((cp1 = getstring(buf, sizeof(buf))) == NULL)
 			return false;
 		c.re = strtod(cp1, &cp1);
 	} while (*cp1);
 	do {
 		my_strlcpy(prompt_str, _("Enter imaginary part: "), sizeof(prompt_str));
-		if ((cp1 = getstring((char *) scratch, n_tokens * sizeof(token_type))) == NULL)
+		if ((cp1 = getstring(buf, sizeof(buf))) == NULL)
 			return false;
 		c.im = strtod(cp1, &cp1);
 	} while (*cp1);
@@ -215,9 +243,7 @@ int		*np;
 		if (c.im == 0.0 && p.im == 0.0)
 			continue;
 		i += len + 1;
-#if	!SILENT
-		printf(_("Warning: Complex number root taken.\n"));
-#endif
+		debug_string(0, _("Warning: Complex number root taken."));
 		c = complex_pow(c, p);
 		if (*np + 5 - (i - j) > n_tokens) {
 			error_huge();
