@@ -52,11 +52,11 @@ token_type	*equation;
 int		*np, loc, level;
 long		v;
 {
-	int	i, j;
-	int	n;
-	int	op;
-	int	oploc;
-	int	endloc;
+	int		i, j;
+	int		n;
+	int		op;
+	int		oploc;
+	int		endloc;
 	complexs	c;
 
 	if (equation[loc].level < level) {
@@ -298,7 +298,7 @@ d_power:
  * The derivative command.
  */
 int
-derivative(cp)
+derivative_cmd(cp)
 char	*cp;
 {
 	long		v;
@@ -315,11 +315,8 @@ char	*cp;
 		return false;
 	}
 	i = next_espace();
+	er = !solved_equation(cur_equation);
 	if (n_rhs[cur_equation]) {
-		er = !solved_equation(cur_equation);
-		if (er) {
-			printf(_("Warning: Current equation is not a solved equation.\n"));
-		}
 		source = rhs[cur_equation];
 		nps = &n_rhs[cur_equation];
 		dest = rhs[i];
@@ -343,21 +340,18 @@ char	*cp;
 		if (*cp) {
 			order = strtol(cp, &cp, 10);
 		}
-		if (extra_garbage(cp)) {
+		if (*cp || order <= 0) {
+			error(_("The order must be a positive integer."));
 			return false;
 		}
 	}
-	if (order <= 0) {
-		printf(_("Order out of range.\n"));
-		return false;
-	}
 	if (v == 0) {
 		if (!prompt_var(&v)) {
 			return false;
 		}
 	}
 	if (v != MATCH_ANY && !found_var(source, *nps, v)) {
-		printf(_("Variable not found.  Result would be zero.\n"));
+		error(_("Variable not found.  Result would be zero."));
 		return false;
 	}
 	blt(dest, source, *nps * sizeof(token_type));
@@ -365,7 +359,7 @@ char	*cp;
 	simp_loop(dest, np);
 	for (j = 0; j < order; j++) {
 		if (!differentiate(dest, np, v)) {
-			printf(_("Differentiation failed.\n"));
+			error(_("Differentiation failed."));
 			*np = 0;
 			return false;
 		}
@@ -388,7 +382,7 @@ char	*cp;
 		}
 	}
 	cur_equation = i;
-	list_sub(cur_equation);
+	return_result(cur_equation);
 	return true;
 }
 
@@ -410,7 +404,7 @@ char	*cp;
 	}
 	if (n_rhs[cur_equation]) {
 		if (!solved_equation(cur_equation)) {
-			printf(_("Current equation is not solved for a variable.\n"));
+			error(_("Current equation is not solved for a variable."));
 			return false;
 		}
 		source = rhs[cur_equation];
@@ -433,7 +427,7 @@ char	*cp;
 		return false;
 	}
 	if (!found_var(source, n, v)) {
-		printf(_("Variable not found.\n"));
+		error(_("Variable not found."));
 		return false;
 	}
 	i = next_espace();
@@ -441,12 +435,12 @@ char	*cp;
 	n_rhs[i] = n;
 	simp_loop(rhs[i], &n_rhs[i]);
 	if (!differentiate(rhs[i], &n_rhs[i], v)) {
-		printf(_("Differentiation failed.\n"));
+		error(_("Differentiation failed."));
 		return false;
 	}
 	simpa_side(rhs[i], &n_rhs[i], false, true);
 	if (!found_var(rhs[i], n_rhs[i], v)) {
-		printf(_("There are no extrema for this equation.\n"));
+		error(_("There are no extrema for this equation."));
 		return false;
 	}
 	n_lhs[i] = 1;
@@ -458,11 +452,11 @@ char	*cp;
 	want.kind = VARIABLE;
 	want.token.variable = v;
 	if (!solve_sub(&want, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i])) {
-		printf(_("Solve failed.\n"));
+		error(_("Solve failed."));
 		return false;
 	}
 	simpa_side(rhs[i], &n_rhs[i], false, false);
-	list_sub(cur_equation);
+	return_result(cur_equation);
 	return true;
 }
 
@@ -470,20 +464,20 @@ char	*cp;
  * The taylor command.
  */
 int
-taylor(cp)
+taylor_cmd(cp)
 char	*cp;
 {
-	long	v;
-	int	i, j, k;
-	int	i1, j1;
-	int	level;
-	int	n;
-	int	order;
+	long		v;
+	int		i, j, k;
+	int		i1, j1;
+	int		level;
+	int		n;
+	int		order;
 	token_type	*ep;
-	double	d;
-	char	*cp1;
-	int	our;
-	int	our_nlhs, our_nrhs;
+	double		d;
+	char		*cp1, buf[MAX_CMD_LEN];
+	int		our;
+	int		our_nlhs, our_nrhs;
 
 	if (notdefined(cur_equation)) {
 		return false;
@@ -500,7 +494,7 @@ char	*cp;
 			return false;
 	}
 	if (!found_var(rhs[cur_equation], n_rhs[cur_equation], v)) {
-		printf(_("Variable not found.\n"));
+		error(_("Variable not found."));
 		return false;
 	}
 	i = next_espace();
@@ -509,14 +503,14 @@ char	*cp;
 	n_rhs[i] = 0;
 	our = next_espace();
 	n_lhs[i] = 0;
-	list_var(v, false);
-	printf(_("Taylor approximation of current equation about %s = point.\n"), var_str);
 	blt(rhs[our], rhs[cur_equation], n_rhs[cur_equation] * sizeof(token_type));
 	our_nrhs = n_rhs[cur_equation];
 	if (!differentiate(rhs[our], &our_nrhs, v)) {
-		printf(_("Differentiation failed.\n"));
+		error(_("Differentiation failed."));
 		return false;
 	}
+	list_var(v, false);
+	printf(_("Taylor approximation of current equation about %s = point.\n"), var_str);
 oops:
 	my_strlcpy(prompt_str, _("Enter point: "), sizeof(prompt_str));
 	if (!get_expr(lhs[our], &our_nlhs)) {
@@ -524,7 +518,7 @@ oops:
 		goto oops;
 	}
 	my_strlcpy(prompt_str, _("Enter order (number of derivatives to take): "), sizeof(prompt_str));
-	if ((cp1 = getstring((char *) scratch, n_tokens * sizeof(token_type))) == NULL)
+	if ((cp1 = getstring(buf, sizeof(buf))) == NULL)
 		return false;
 	if (*cp1 == '\0') {
 		printf(_("Derivatives will be taken until they reach zero.\n"));
@@ -532,7 +526,7 @@ oops:
 	} else {
 		order = strtol(cp1, &cp, 10);
 		if (cp == cp1 || order < 0) {
-			printf(_("Positive integer required.\n"));
+			error(_("Positive integer required."));
 			return false;
 		}
 	}
@@ -606,7 +600,7 @@ loop_again:
 	if (n < order) {
 		if (n > 0) {
 			if (!differentiate(rhs[our], &our_nrhs, v)) {
-				printf(_("Differentiation failed.\n"));
+				error(_("Differentiation failed."));
 				return false;
 			}
 		}
@@ -631,7 +625,7 @@ loop_again:
 	}
 	n_lhs[i] = n_lhs[cur_equation];
 	cur_equation = i;
-	list_sub(cur_equation);
+	return_result(cur_equation);
 	return true;
 }
 
@@ -658,7 +652,7 @@ char	*cp;
 		return false;
 	}
 	if (!solved_equation(cur_equation)) {
-		printf(_("Current equation is not solved for a variable.\n"));
+		error(_("Current equation is not solved for a variable."));
 		return false;
 	}
 	blt(&solved_v, lhs[cur_equation], sizeof(token_type));
@@ -674,7 +668,7 @@ char	*cp;
 		}
 	}
 	if (!found_var(rhs[cur_equation], n_rhs[cur_equation], v)) {
-		printf(_("Variable not found.\n"));
+		error(_("Variable not found."));
 		return false;
 	}
 	want.level = 1;
@@ -741,7 +735,7 @@ char	*cp;
 	if (!found_den || (infinity_flag && !complex_den)) {
 		subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], lhs[i], n, v);
 		simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
-		list_sub(cur_equation);
+		return_result(cur_equation);
 		return true;
 	}
 	if (infinity_flag) {
@@ -759,7 +753,7 @@ char	*cp;
 		subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], tlhs, n_tlhs, v);
 	}
 	if (!solve_sub(&want, 1, lhs[cur_equation], &n_lhs[cur_equation], rhs[cur_equation], &n_rhs[cur_equation])) {
-		printf(_("Can't take limit of this equation because solve failed.\n"));
+		error(_("Can't take limit of this equation because solve failed."));
 		return false;
 	}
 	blt(lhs[cur_equation], lhs[i], n * sizeof(token_type));
@@ -768,12 +762,12 @@ char	*cp;
 	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
 	symb_flag = false;
 	if (!solve_sub(&solved_v, 1, lhs[cur_equation], &n_lhs[cur_equation], rhs[cur_equation], &n_rhs[cur_equation])) {
-		printf(_("Can't take limit of this equation because solve failed.\n"));
+		error(_("Can't take limit of this equation because solve failed."));
 		return false;
 	}
 	symb_flag = true;
 	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
 	symb_flag = false;
-	list_sub(cur_equation);
+	return_result(cur_equation);
 	return true;
 }
