@@ -38,8 +38,8 @@ int
 factor_one(start)
 double	start;
 {
-	int		i;
-	double		ii;
+	int	i;
+	double	ii;
 
 	uno = 0;
 	nn = start;
@@ -79,10 +79,7 @@ static void
 try_factor(arg)
 double	arg;
 {
-	for (;;) {
-		if (fmod(nn, arg) != 0.0) {
-			break;
-		}
+	while (fmod(nn, arg) == 0.0) {
 		if (uno > 0 && unique[uno-1] == arg) {
 			ucnt[uno-1]++;
 		} else {
@@ -116,8 +113,11 @@ multiply_out_unique()
 }
 
 /*
- * Factor integers in an equation side.
+ * Factor integers in an expression.
+ *
+ * Return true if expression was modified.
  */
+int
 factor_int(equation, np)
 token_type	*equation;
 int		*np;
@@ -125,58 +125,63 @@ int		*np;
 	int	i, j;
 	int	xsize;
 	int	level;
+	int	modified;
 
+	modified = false;
 	for (i = 0; i < *np; i += 2) {
-		if (equation[i].kind == CONSTANT) {
-			if (factor_one(equation[i].token.constant) && uno > 0) {
-				level = equation[i].level;
-				if (uno > 1 && *np > 1)
-					level++;
-				xsize = -2;
-				for (j = 0; j < uno; j++) {
-					if (ucnt[j] > 1)
-						xsize += 4;
-					else
-						xsize += 2;
-				}
-				if (*np + xsize > n_tokens)
-					error_huge();
-				for (j = 0; j < uno; j++) {
-					if (ucnt[j] > 1)
-						xsize = 4;
-					else
-						xsize = 2;
-					if (j == 0)
-						xsize -= 2;
-					if (xsize > 0) {
-						blt(&equation[i+xsize], &equation[i], (*np - i) * sizeof(token_type));
-						*np += xsize;
-						if (j > 0) {
-							i++;
-							equation[i].kind = OPERATOR;
-							equation[i].level = level;
-							equation[i].token.operatr = TIMES;
-							i++;
-						}
-					}
-					equation[i].kind = CONSTANT;
-					equation[i].level = level;
-					equation[i].token.constant = unique[j];
-					if (ucnt[j] > 1) {
-						equation[i].level = level + 1;
+		if (equation[i].kind == CONSTANT && factor_one(equation[i].token.constant) && uno > 0) {
+			if (uno == 1 && ucnt[0] <= 1)
+				continue;	/* prime number */
+			level = equation[i].level;
+			if (uno > 1 && *np > 1)
+				level++;
+			xsize = -2;
+			for (j = 0; j < uno; j++) {
+				if (ucnt[j] > 1)
+					xsize += 4;
+				else
+					xsize += 2;
+			}
+			if (*np + xsize > n_tokens) {
+				error_huge();
+			}
+			for (j = 0; j < uno; j++) {
+				if (ucnt[j] > 1)
+					xsize = 4;
+				else
+					xsize = 2;
+				if (j == 0)
+					xsize -= 2;
+				if (xsize > 0) {
+					blt(&equation[i+xsize], &equation[i], (*np - i) * sizeof(token_type));
+					*np += xsize;
+					if (j > 0) {
 						i++;
 						equation[i].kind = OPERATOR;
-						equation[i].level = level + 1;
-						equation[i].token.operatr = POWER;
+						equation[i].level = level;
+						equation[i].token.operatr = TIMES;
 						i++;
-						equation[i].level = level + 1;
-						equation[i].kind = CONSTANT;
-						equation[i].token.constant = ucnt[j];
 					}
 				}
+				equation[i].kind = CONSTANT;
+				equation[i].level = level;
+				equation[i].token.constant = unique[j];
+				if (ucnt[j] > 1) {
+					equation[i].level = level + 1;
+					i++;
+					equation[i].kind = OPERATOR;
+					equation[i].level = level + 1;
+					equation[i].token.operatr = POWER;
+					i++;
+					equation[i].level = level + 1;
+					equation[i].kind = CONSTANT;
+					equation[i].token.constant = ucnt[j];
+				}
 			}
+			modified = true;
 		}
 	}
+	return modified;
 }
 
 /*
@@ -1105,15 +1110,15 @@ fpower_sub(equation, np, loc, i1, n1, i2, n2, level)
 token_type	*equation;
 int		*np, loc, i1, n1, i2, n2, level;
 {
-	int	e1, e2;
-	int	op1, op2;
-	int	pop1;
-	int	start2;
-	int	i, j, k;
-	int	b1, b2;
-	int	len;
-	int	diff_sign;
-	int	all_divide;
+	int		e1, e2;
+	int		op1, op2;
+	int		pop1;
+	int		start2;
+	int		i, j, k;
+	int		b1, b2;
+	int		len;
+	int		diff_sign;
+	int		all_divide;
 	token_type	temp;
 
 	e1 = i1 + n1;
