@@ -1,5 +1,5 @@
 /*
- * Algebraic manipulator numeric GCD routines.
+ * Floating point GCD routines.
  *
  * Copyright (c) 1996 George Gesslein II.
  */
@@ -100,9 +100,9 @@ double	*d2p;	/* returned denominator */
 		d2 = 1.0 / divisor;
 		d1 = my_round(d1);
 		d2 = my_round(d2);
-		if (fabs(d1) >= 1.0e10)
+		if (fabs(d1) >= 1.0e12)
 			return false;
-		if (d2 >= 1.0e10)
+		if (d2 >= 1.0e12)
 			return false;
 		if (d2 < 1.5)
 			return false;
@@ -207,26 +207,3 @@ int		*np;
 		}
 	}
 }
-
-/*
- * Output GCD and LCM of doubles d1 and d2.
- *
- * Return true if the GCD was found and displayed.
- */
-int
-ngcd(d1, d2)
-double	d1, d2;
-{
-	double	d3;
-
-	d1 = fabs(d1);
-	d2 = fabs(d2);
-	d3 = gcd(d1, d2);
-	if (d3 <= 0.0) {
-		printf(_("No GCD found.\n"));
-		return false;
-	}
-	printf(_("Greatest Common Divisor (GCD) = %.14g\n"), d3);
-	printf(_("Least Common Multiple (LCM) = %.14g\n"), (d1 * d2) / d3);
-	return true;
-}
