@@ -6,7 +6,7 @@
 #include "includes.h"
 
 int		n_tokens = DEFAULT_N_TOKENS;	/* maximum size of expressions */
-int		n_equations;
+int		n_equations;			/* number of equation spaces allocated */
 int		cur_equation;			/* current equation space number (origin 0) */
 
 int		n_lhs[N_EQUATIONS];
@@ -52,6 +52,7 @@ char		prompt_str[MAX_VAR_LEN+80];	/* temp storage for prompt strings */
 #if	CYGWIN
 char		*dir_path;			/* directory path to the executable */
 #endif
+char		*prog_name = "mathomatic";	/* name of this program */
 
 /* The following are for integer factoring (filled by factor_one()). */
 double		unique[64];		/* storage for the unique factors */
