@@ -8,32 +8,43 @@
 
 #include "includes.h"
 
+/*
+ * The following structure is used for each Mathomatic command.
+ */
+typedef	struct {
+	char	*name;		/* command name to be typed by user */
+	int	(*func)();	/* function that handles this command */
+				/* function is passed a char pointer and returns true if successful */
+	char	*usage;		/* command syntax */
+	char	*info;		/* one line description of command */
+} com_type;
+
 /*
  * The Mathomatic command table follows.  It should be in alphabetical order.
  */
-com_type com_list[] = {
+static com_type com_list[] = {
 {	"calculate",	calculate,	"[variable number-of-iterations]",		"Temporarily plug in values for variables and approximate.  Option to iterate." },
 {	"clear",	clear,		"[equation-number-range]",			"Delete equations so equation space can be reused." },
 {	"code",		code_cmd,	"[\"c\" or \"java\" or \"integer\"] [equation-number-range]",	"Output C or Java code for the specified equations." },
 {	"compare",	compare_cmd,	"equation-number [\"with\" equation-number]",	"Compare two equations to see if they are the same." },
 {	"copy",		copy_cmd,	"[equation-number-range]",			"Duplicate the specified equations." },
-{	"derivative",	derivative,	"variable or \"all\" [order]",			"Symbolically differentiate the current equation, order times." },
+{	"derivative",	derivative_cmd,	"variable or \"all\" [order]",			"Symbolically differentiate the current equation, order times." },
 {	"divide",	divide_cmd,	"[variable]",					"Prompt for 2 polynomials/numbers and divide.  Display result and GCD." },
 #if	(UNIX || CYGWIN) && !SECURE
-{	"edit",		edit,		"[file-name]",					"Edit your equations or an input file." },
+{	"edit",		edit_cmd,	"[file-name]",					"Edit your equations or an input file." },
 #endif
-{	"eliminate",	eliminate,	"variables or \"all\" [\"using\" equation-number]",	"Substitute the specified variables with solved equations." },
+{	"eliminate",	eliminate_cmd,	"variables or \"all\" [\"using\" equation-number]",	"Substitute the specified variables with solved equations." },
 {	"extrema",	extrema_cmd,	"variable",					"Find the minimums and maximums of the current equation." },
-{	"factor",	factor,		"[\"number\" [integers]] or [equation-number-range] [variables]",	"Factor integers or equations.  For polynomial factoring, use \"simplify\"." },
+{	"factor",	factor_cmd,	"[\"number\" [integers]] or [equation-number-range] [variables]",	"Factor integers or equations.  For polynomial factoring, use \"simplify\"." },
 {	"flist",	flist_cmd,	"[\"factor\"] [equation-number-range]",		"Display equations in fraction format.  Option to factor integers." },
-{	"help",		help,		"[topic or command-name]",			"Short, built-in help." },
-{	"imaginary",	imaginary,	"[equation-number]",				"Copy the imaginary part of an equation (see the \"real\" command)." },
-{	"integrate",	integrate_cmd,	"variable [order]",				"Symbolically integrate the current equation, order times." },
+{	"help",		help_cmd,	"[topic or command-name]",			"Short, built-in help." },
+{	"imaginary",	imaginary_cmd,	"[equation-number]",				"Copy the imaginary part of an equation (see the \"real\" command)." },
+{	"integrate",	integrate_cmd,	"variable [order]",				"Symbolically integrate polynomials, order times." },
 {	"laplace",	laplace_cmd,	"[\"inverse\"] variable",			"Compute the Laplace or inverse Laplace transform of the current equation." },
 {	"limit",	limit_cmd,	"variable expression",				"Take the limit of the current equation as variable goes to expression." },
 {	"list",		list_cmd,	"[\"export\"] [equation-number-range]",		"Display equations in single line format.  Option for exportable format." },
-{	"nintegrate",	nintegrate,	"[\"trapezoidal\"] variable",			"Approximate the definite integral with respect to variable."},
-{	"optimize",	optimize_cmd,	"[equation-number]",				"Split up an equation into smaller multiple equations." },
+{	"nintegrate",	nintegrate_cmd,	"[\"trapezoidal\"] variable [partitions]",	"Approximate the definite integral using Simpson's rule."},
+{	"optimize",	optimize_cmd,	"[equation-number-range]",			"Split up an equation into smaller multiple equations." },
 {	"pause",	pause_cmd,	"[text]",					"Wait for user to press the Enter key.  Optionally display a message." },
 {	"product",	product_cmd,	"variable low high",				"Compute the product of the current equation." },
 #if	READLINE
@@ -43,18 +54,18 @@ com_type com_list[] = {
 #if	!SECURE
 {	"read",		read_cmd,	"file-name",					"Read in a text file as if it was typed in." },
 #endif
-{	"real",		real,		"[equation-number]",				"Copy the real part of an equation (see the \"imaginary\" command)." },
-{	"replace",	replace,	"[\"constants\"] [variables [\"with\" expression]]",	"Substitute variables in the current equation with expressions." },
+{	"real",		real_cmd,	"[equation-number]",				"Copy the real part of an equation (see the \"imaginary\" command)." },
+{	"replace",	replace_cmd,	"[\"constants\"] [variables [\"with\" expression]]",	"Substitute variables in the current equation with expressions." },
 {	"roots",	roots_cmd,	"",						"Display all the roots of a complex number." },
 #if	!SECURE
-{	"save",		save,		"file-name",					"Save all equations in a text file (restore with the \"read\" command)." },
+{	"save",		save_cmd,	"file-name",					"Save all equations in a text file (restore with the \"read\" command)." },
 #endif
 {	"set",		set_cmd,	"[\"no\"] [option]",				"Set various session options." },
-{	"simplify",	simplify,	"[\"symbolic\" or \"quick\" or \"poly\"] [equation-number-range]",	"Completely simplify equations." },
+{	"simplify",	simplify_cmd,	"[\"symbolic\" or \"quick\" or \"poly\"] [equation-number-range]",	"Completely simplify equations." },
 {	"sum",		sum_cmd,	"variable low high",				"Compute the summation of the current equation." },
-{	"tally",	tally,		"",						"Prompt for and add numerical entries.  Show running total." },
-{	"taylor",	taylor,		"variable",					"Compute the Taylor series expansion of the current equation." },
-{	"unfactor",	unfactor,	"[\"fully\"] [equation-number-range]",		"Expand equations." }
+{	"tally",	tally_cmd,	"",						"Prompt for and add numerical entries.  Show running total." },
+{	"taylor",	taylor_cmd,	"variable",					"Compute the Taylor series expansion of the current equation." },
+{	"unfactor",	unfactor_cmd,	"[\"fully\"] [equation-number-range]",		"Expand equations." }
 };
 
 int	point_flag;	/* point to error if true */
@@ -62,6 +73,7 @@ int	point_flag;	/* point to error if true */
 /*
  * Process a line of input to Mathomatic.
  * It may be a command, an equation, etc.
+ *
  * Return true if successful.
  */
 int
@@ -79,6 +91,9 @@ char	*cp;
 	FILE	*fp;
 	int	append_flag;
 
+	if (cp == NULL) {
+		return false;
+	}
 	set_sign_array();
 	cp_start = cp;
 	set_error_level(cp);
@@ -86,15 +101,11 @@ char	*cp;
 	if (*cp == '#') {
 		cp++;
 		if (isascii(*cp) && isdigit(*cp)) {
-			i = atoi(cp) - 1;
+			i = strtol(cp, &cp, 10) - 1;
 			if (i < 0 || i >= n_equations) {
-#if	!SILENT
-				put_up_arrow((int) (cp - cp_start));
-				printf(_("Equation number out of range.\n"));
-#endif
+				error(_("Equation number out of range."));
 				return false;
 			}
-			cp = skip_num(cp);
 			if (*cp == ':') {
 				cp++;
 			}
@@ -139,9 +150,7 @@ char	*cp;
 			cp1 = skip_space(cp1);
 			input_column += (cp1 - cp_start);
 			if (my_strlcpy(buf2, cp1, sizeof(buf2)) >= sizeof(buf2)) {
-#if	!SILENT
-				printf(_("Command line too long.\n"));
-#endif
+				error(_("Command line too long."));
 				return false;
 			}
 			fp = NULL;
@@ -214,7 +223,7 @@ e_parse:
 				    || (n_rhs[n] == 1 && (rhs[n][0].kind != CONSTANT || rhs[n][0].token.constant == 0.0)
 				    && !(rhs[n][0].kind == VARIABLE && (rhs[n][0].token.variable & VAR_MASK) <= SIGN))) {
 					if (solve(n, cur_equation)) {
-						list_sub(cur_equation);
+						return_result(cur_equation);
 						return true;
 					}
 					return false;
@@ -258,6 +267,7 @@ e_parse:
 #if	!SECURE
 /*
  * Execute a system command.
+ *
  * Returns exit value of command (0 if no error).
  */
 int
@@ -273,27 +283,28 @@ char	*cp;
 }
 #endif
 
+/*
+ * Parse a variable name, with error messages.
+ *
+ * Return new position in string or NULL if error.
+ */
 char	*
 parse_var2(vp, cp)
-long	*vp;
-char	*cp;
+long	*vp;	/* pointer to result */
+char	*cp;	/* pointer to variable name string */
 {
 	char	*cp1;
 
 	cp = skip_space(cp);
 	cp = parse_var(vp, cp);
 	if (cp == NULL) {
-#if	!SILENT
-		printf(_("Invalid variable.\n"));
-#endif
+		error(_("Invalid variable."));
 		return NULL;
 	}
 	if (*cp) {
 		cp1 = skip_space(cp);
 		if (cp == cp1) {
-#if	!SILENT
-			printf(_("Please separate variables with spaces.\n"));
-#endif
+			error(_("Please separate variables with spaces."));
 			return NULL;
 		}
 		return cp1;
@@ -301,8 +312,11 @@ char	*cp;
 	return cp;
 }
 
+/*
+ * Set point_flag if pointing to the input error works for the passed string.
+ */
 set_error_level(cp)
-char	*cp;
+char	*cp;	/* input string */
 {
 	char	*cp1;
 
@@ -337,9 +351,7 @@ int	cnt;	/* position of error, relative to "input_column" */
 {
 	int	 i;
 
-	if (piping_in_flag)
-		return;
-	if (!point_flag)
+	if (piping_in_flag || !point_flag)
 		return;
 	for (i = 0; i < (input_column + cnt); i++) {
 		printf(" ");
@@ -359,7 +371,8 @@ char	*cp;
 }
 
 /*
- * Process an equation number range.
+ * Process an equation number range given on the command line.
+ *
  * Return true if successful,
  * with the starting equation number in "*ip"
  * and ending equation number in "*jp".
@@ -375,49 +388,48 @@ int	*ip, *jp;
 		*cpp = skip_param(*cpp);
 		*ip = 0;
 		*jp = n_equations - 1;
-		goto test_range;
-	}
-	if (isascii(**cpp) && isdigit(**cpp)) {
-		*ip = atoi(*cpp) - 1;
 	} else {
-		*ip = cur_equation;
-	}
-	if (*ip < 0 || *ip >= n_equations) {
-		printf(_("Invalid equation number.\n"));
-		return false;
-	}
-	*cpp = skip_num(*cpp);
-	if (**cpp != '-') {
-		if (notdefined(*ip)) {
+		if (isascii(**cpp) && isdigit(**cpp)) {
+			*ip = atoi(*cpp) - 1;
+		} else {
+			*ip = cur_equation;
+		}
+		if (*ip < 0 || *ip >= n_equations) {
+			error(_("Invalid equation number."));
 			return false;
 		}
-		*jp = *ip;
-		return true;
+		*cpp = skip_num(*cpp);
+		if (**cpp != '-') {
+			if (notdefined(*ip)) {
+				return false;
+			}
+			*jp = *ip;
+			return true;
+		}
+		(*cpp)++;
+		*cpp = skip_space(*cpp);
+		if (isascii(**cpp) && isdigit(**cpp)) {
+			*jp = atoi(*cpp) - 1;
+		} else {
+			*jp = cur_equation;
+		}
+		if (*jp < 0 || *jp >= n_equations) {
+			error(_("Invalid equation number."));
+			return false;
+		}
+		if (*jp < *ip) {
+			i = *ip;
+			*ip = *jp;
+			*jp = i;
+		}
+		*cpp = skip_num(*cpp);
 	}
-	(*cpp)++;
-	*cpp = skip_space(*cpp);
-	if (isascii(**cpp) && isdigit(**cpp)) {
-		*jp = atoi(*cpp) - 1;
-	} else {
-		*jp = cur_equation;
-	}
-	if (*jp < 0 || *jp >= n_equations) {
-		printf(_("Invalid equation number.\n"));
-		return false;
-	}
-	if (*jp < *ip) {
-		i = *ip;
-		*ip = *jp;
-		*jp = i;
-	}
-	*cpp = skip_num(*cpp);
-test_range:
 	for (i = *ip; i <= *jp; i++) {
 		if (n_lhs[i] > 0) {
 			return true;
 		}
 	}
-	printf(_("No equations defined in specified range.\n"));
+	error(_("No equations defined in specified range."));
 	return false;
 }
 
@@ -431,7 +443,7 @@ char	*cp;
 {
 	cp = skip_space(cp);
 	if (*cp) {
-		printf(_("ERROR: Extra characters on command line.\n"));
+		error(_("Extra characters on command line."));
 		return true;
 	}
 	return false;
@@ -519,7 +531,7 @@ char	*cp1, *cp2;
 /*
  * The help command.
  */
-help(cp)
+help_cmd(cp)
 char	*cp;
 {
 	int	i, j;
@@ -610,12 +622,11 @@ char	*cp;
 	P("Type \"help equations\" for help with entering equations.");
 	P("Type \"help geometry\" for some commonly used geometric formulas.");
 	P("\"help\" followed by a command name will give info on a command.\n");
-	P("The following is a list of available commands:\n");
+	P("Available commands:");
 	for (i = 0; i < ARR_CNT(com_list); i++) {
-		if (i && (i % 4) == 0)
+		if ((i % 5) == 0)
 			fprintf(gfp, "\n");
-		fprintf(gfp, "%s", com_list[i].name);
-		j = 19 - strlen(com_list[i].name);
+		j = 15 - fprintf(gfp, "%s", com_list[i].name);
 		for (; j > 0; j--)
 			fprintf(gfp, " ");
 	}
