@@ -101,14 +101,14 @@ int		*np;
 int		loc, eloc;
 long		v;
 {
-	int	i, j, k;
-	int	len;
-	int	level;
-	int	mlevel;
-	int	count;
-	int	exp_pos;
+	int		i, j, k;
+	int		len;
+	int		level;
+	int		mlevel;
+	int		count;
+	int		exp_pos;
 	complexs	c;
-	int	div_flag;
+	int		div_flag;
 
 	exp_pos = -1;
 	level = min_level(&equation[loc], eloc - loc);
@@ -305,11 +305,8 @@ char	*cp;
 		return false;
 	}
 	i = next_espace();
+	er = !solved_equation(cur_equation);
 	if (n_rhs[cur_equation]) {
-		er = !solved_equation(cur_equation);
-		if (er) {
-			printf(_("Warning: Current equation is not a solved equation.\n"));
-		}
 		source = rhs[cur_equation];
 		nps = &n_rhs[cur_equation];
 		dest = rhs[i];
@@ -328,7 +325,8 @@ char	*cp;
 		if (*cp) {
 			order = strtol(cp, &cp, 10);
 		}
-		if (extra_garbage(cp)) {
+		if (*cp || order <= 0) {
+			error(_("The order must be a positive integer."));
 			return false;
 		}
 	} else {
@@ -336,10 +334,6 @@ char	*cp;
 			return false;
 		}
 	}
-	if (order <= 0) {
-		printf(_("Order out of range.\n"));
-		return false;
-	}
 	partial_flag = false;
 	uf_simp(source, nps);
 	partial_flag = true;
@@ -348,7 +342,7 @@ char	*cp;
 	*np = *nps;
 	for (j = 0; j < order; j++) {
 		if (!integrate(dest, np, v)) {
-			printf(_("Integration failed.\n"));
+			error(_("Integration failed."));
 			*np = 0;
 			return false;
 		}
@@ -372,7 +366,7 @@ char	*cp;
 	}
 	cur_equation = i;
 	simpa_side(dest, np, false, false);
-	list_sub(cur_equation);
+	return_result(cur_equation);
 	return true;
 }
 
@@ -420,12 +414,12 @@ int		*np;
 int		loc, eloc;
 long		v;
 {
-	int	i, j, k;
-	int	len;
-	int	level, level2;
-	int	mlevel;
-	int	count;
-	int	exp_pos;
+	int		i, j, k;
+	int		len;
+	int		level, level2;
+	int		mlevel;
+	int		count;
+	int		exp_pos;
 	complexs	c;
 
 	exp_pos = -1;
@@ -776,13 +770,13 @@ char	*cp;
 	if (inverse_flag) {
 		if (!poly_in_v(dest, *np, v, true)
 		    || !inv_laplace(dest, np, v)) {
-			printf(_("Inverse Laplace failed.\n"));
+			error(_("Inverse Laplace failed."));
 			*np = 0;
 			return false;
 		}
 	} else {
 		if (!laplace(dest, np, v)) {
-			printf(_("Laplace failed.\n"));
+			error(_("Laplace failed."));
 			*np = 0;
 			return false;
 		}
@@ -793,7 +787,7 @@ char	*cp;
 	}
 	cur_equation = i;
 	simp_loop(dest, np);
-	list_sub(cur_equation);
+	return_result(cur_equation);
 	return true;
 }
 
@@ -801,15 +795,13 @@ char	*cp;
  * Numerical integrate command.
  */
 int
-nintegrate(cp)
+nintegrate_cmd(cp)
 char	*cp;
 {
 	long		v;
-	long		l;
 	int		i, j, k;
 	int		i1, j1;
 	int		level;
-	int		er;
 	int		iterations;
 	int		first_size;
 	token_type	*ep;
@@ -832,17 +824,15 @@ char	*cp;
 		if (cp == NULL) {
 			return false;
 		}
-		if (strncasecmp(cp, "trap", 4) == 0) {
-			trap_flag = true;
-		} else if (extra_garbage(cp)) {
+		if (*cp) {
+			iterations = strtol(cp, &cp, 10);
+		}
+		if (*cp || iterations <= 0 || (iterations % 2) != 0) {
+			error(_("Number of partitions must be a positive, even integer."));
 			return false;
 		}
 	}
 	i = next_espace();
-	er = !solved_equation(cur_equation);
-	if (er) {
-		printf(_("Warning: Current equation is not a solved equation.\n"));
-	}
 	singularity = false;
 	for (j = 1; j < n_rhs[cur_equation]; j += 2) {
 		if (rhs[cur_equation][j].token.operatr == DIVIDE) {
@@ -865,7 +855,7 @@ char	*cp;
 	subst_constants(tlhs, &n_tlhs);
 	for (j = 0; j < n_tlhs; j += 2) {
 		if (tlhs[j].kind == CONSTANT && !isfinite(tlhs[j].token.constant)) {
-			printf(_("Error: Bound contains infinity.\n"));
+			error(_("Error: Bound contains infinity."));
 			return false;
 		}
 	}
@@ -876,18 +866,17 @@ char	*cp;
 	subst_constants(trhs, &n_trhs);
 	for (j = 0; j < n_trhs; j += 2) {
 		if (trhs[j].kind == CONSTANT && !isfinite(trhs[j].token.constant)) {
-			printf(_("Error: Bound contains infinity.\n"));
+			error(_("Error: Bound contains infinity."));
 			return false;
 		}
 	}
 	if ((n_tlhs + 1 + n_trhs + 2) > n_tokens) {
 		error_huge();
 	}
-	printf(_("Approximating the definite integral using "));
 	if (trap_flag) {
-		printf(_("the trapezoidal method...\n"));
+		debug_string(0, _("Approximating the definite integral using the trapezoidal method..."));
 	} else {
-		printf(_("Simpson's rule...\n"));
+		debug_string(0, _("Approximating the definite integral using Simpson's rule..."));
 	}
 	for (j = 0; j < n_trhs; j++)
 		trhs[j].level += 2;
@@ -914,14 +903,6 @@ char	*cp;
 	rhs[i][0].token.constant = 0.0;
 	n_rhs[i] = 1;
 	for (j = 0; j <= iterations; j++) {
-		if (j > 0 && color_flag
-		    && ((j % (iterations / 10)) == 0 || j == iterations)) {
-			printf("%ld%s", (j * 100L) / iterations, "% complete.");
-			if (j == iterations)
-				printf("\n");
-			else
-				printf("\r");
-		}
 		if ((n_rhs[i] + 1 + n_rhs[cur_equation]) > n_tokens)
 			error_huge();
 		for (k = 0; k < n_rhs[i]; k++)
@@ -1005,7 +986,7 @@ char	*cp;
 					first_size = 4;
 			} else {
 				if ((n_rhs[i] / 8) >= first_size) {
-					printf(_("Integration failed.\n"));
+					error(_("Integration failed."));
 					return false;
 				}
 			}
@@ -1040,23 +1021,16 @@ char	*cp;
 	elim_loop(rhs[i], &n_rhs[i]);
 	ufactor(rhs[i], &n_rhs[i]);
 	simp_divide(rhs[i], &n_rhs[i], 0L);
+	debug_string(0, _("Integration successful."));
 	if (n_rhs[i] == 1) {
-		printf(_("Integration successful.  Result = "));
-		list_proc(rhs[i], n_rhs[i]);
-		printf("\n");
+		fprintf(gfp, _("Result = "));
+		list_proc(rhs[i], n_rhs[i], false);
+		fprintf(gfp, "\n");
 		return true;
 	}
 	blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
 	n_lhs[i] = n_lhs[cur_equation];
-	if (!er) {
-		l = lhs[i][0].token.variable;
-		l -= PRIME_INCREMENT;
-		if (l > 0) {
-			lhs[i][0].token.variable = l;
-		}
-	}
-	printf(_("Integration successful.\n"));
 	cur_equation = i;
-	list_sub(cur_equation);
+	return_result(cur_equation);
 	return true;
 }
