@@ -21,7 +21,7 @@ main(int argc, char **argv)
 	gfp = stdout;
 
 	if (!init_mem()) {
-		fprintf(stderr, _("Not enough memory.\n"));
+		fprintf(stderr, "Not enough memory.\n");
 		exit(1);
 	}
 	if ((i = setjmp(jmp_save)) != 0) {
