@@ -75,6 +75,8 @@ default_color()
 
 /*
  * Display the equation stored in equation space "n" in single line format.
+ *
+ * Return true if successful.
  */
 int
 list1_sub(n, export_flag)
@@ -108,6 +110,8 @@ int	export_flag;	/* true for exportable format (readable by other math programs)
 
 /*
  * Display the equation stored in equation space "n".
+ *
+ * Return true if successful.
  */
 int
 list_sub(n)
@@ -117,11 +121,10 @@ int	n;	/* equation number */
 		return false;
 	if (groupall) {
 		group_sub(n);
-		flist_sub(n);
+		return flist_sub(n);
 	} else {
-		list1_sub(n, false);
+		return list1_sub(n, false);
 	}
-	return true;
 }
 
 #if	!SILENT
@@ -255,16 +258,17 @@ int	c_flag;
 
 /*
  * Display an expression in single line format.
+ * Use color if available.
  *
  * Return number of characters output (excluding escape sequences).
  */
 int
 list_proc(equation, n, export_flag)
-token_type	*equation;	/* expression */
+token_type	*equation;	/* expression pointer */
 int		n;		/* length of expression */
-int		export_flag;
+int		export_flag;	/* flag for exportable format (usually false) */
 {
-	int	i,j,k,l;
+	int	i, j, k, i1;
 	int	min1;
 	int	cur_level;
 	char	*cp;
@@ -275,7 +279,7 @@ int		export_flag;
 	for (i = 0; i < n; i++) {
 		j = cur_level - equation[i].level;
 		k = abs(j);
-		for (l = 1; l <= k; l++) {
+		for (i1 = 1; i1 <= k; i1++) {
 			if (j > 0) {
 				cur_level--;
 				len += fprintf(gfp, ")");
@@ -288,7 +292,7 @@ int		export_flag;
 		}
 		switch (equation[i].kind) {
 		case CONSTANT:
-			if (high_prec) {
+			if (high_prec || export_flag) {
 				len += fprintf(gfp, "%.20g", equation[i].token.constant);
 			} else if (finance_option) {
 				len += fprintf(gfp, "%.2f", equation[i].token.constant);
@@ -382,7 +386,7 @@ int	export_flag;
  */
 char *
 list_exp(equation, n, export_flag)
-token_type	*equation;	/* expression */
+token_type	*equation;	/* expression pointer */
 int		n;		/* length of expression */
 int		export_flag;
 {
@@ -408,12 +412,12 @@ int		export_flag;
  */
 int
 list_string_sub(equation, n, string, export_flag)
-token_type	*equation;	/* expression */
+token_type	*equation;	/* expression pointer */
 int		n;		/* length of expression */
 char		*string;	/* buffer or NULL pointer */
 int		export_flag;
 {
-	int	i,j,k,l;
+	int	i, j, k, i1;
 	int	min1;
 	int	cur_level;
 	char	*cp;
@@ -427,7 +431,7 @@ int		export_flag;
 	for (i = 0; i < n; i++) {
 		j = cur_level - equation[i].level;
 		k = abs(j);
-		for (l = 1; l <= k; l++) {
+		for (i1 = 1; i1 <= k; i1++) {
 			if (j > 0) {
 				cur_level--;
 				if (string)
@@ -442,7 +446,7 @@ int		export_flag;
 		}
 		switch (equation[i].kind) {
 		case CONSTANT:
-			if (high_prec) {
+			if (high_prec || export_flag) {
 				len += snprintf(buf, sizeof(buf), "%.20g", equation[i].token.constant);
 			} else if (finance_option) {
 				len += snprintf(buf, sizeof(buf), "%.2f", equation[i].token.constant);
@@ -506,8 +510,8 @@ int		export_flag;
  */
 int
 int_expr(equation, n)
-token_type	*equation;
-int		n;
+token_type	*equation;	/* expression pointer */
+int		n;		/* length of expression */
 {
 	int	i;
 
@@ -536,21 +540,43 @@ int		n;
 }
 
 /*
- * Display C or Java code for an expression.
+ * Display an equation as C or Java code.
  */
+list_c_equation(en, java_flag, int_flag)
+int	en;		/* equation number */
+int	java_flag;
+int	int_flag;	/* integer arithmetic flag */
+{
+	if (n_lhs[en] <= 0)
+		return;
+	list_c_code(lhs[en], n_lhs[en], java_flag, int_flag);
+	if (n_rhs[en]) {
+		fprintf(gfp, " = ");
+		list_c_code(rhs[en], n_rhs[en], java_flag, int_flag);
+	}
+	fprintf(gfp, ";\n");
+}
+
+/*
+ * Display C or Java code for an expression.
+ *
+ * Return length of output (number of characters).
+ */
+int
 list_c_code(equation, n, java_flag, int_flag)
-token_type	*equation;	/* expression */
+token_type	*equation;	/* expression pointer */
 int		n;		/* length of expression */
 int		java_flag;	/* if true, generate jave code */
 int		int_flag;	/* integer arithmetic flag */
 {
-	int	i,j,k,l;
+	int	i, j, k, i1, i2;
 	int	min1;
 	int	cur_level;
 	char	*cp;
-	int	i1;
 	char	buf[50];
+	int	len;
 
+	len = 0;
 	min1 = min_level(equation, n);
 	if (n > 1)
 		min1--;
@@ -558,49 +584,53 @@ int		int_flag;	/* integer arithmetic flag */
 	for (i = 0; i < n; i++) {
 		j = cur_level - equation[i].level;
 		k = abs(j);
-		for (l = 1; l <= k; l++) {
+		for (i1 = 1; i1 <= k; i1++) {
 			if (j > 0) {
 				cur_level--;
 				fprintf(gfp, ")");
+				len++;
 			} else {
 				cur_level++;
-				for (i1 = i + 1; i1 < n && equation[i1].level >= cur_level; i1 += 2) {
-					if (equation[i1].level == cur_level) {
-						switch (equation[i1].token.operatr) {
+				for (i2 = i + 1; i2 < n && equation[i2].level >= cur_level; i2 += 2) {
+					if (equation[i2].level == cur_level) {
+						switch (equation[i2].token.operatr) {
 						case POWER:
-							if (equation[i1-1].level == cur_level
-							    && equation[i1+1].level == cur_level
-							    && equation[i1+1].kind == CONSTANT
-							    && equation[i1+1].token.constant == 2.0) {
-								equation[i1].token.operatr = TIMES;
-								equation[i1+1].kind = equation[i1-1].kind;
-								equation[i1+1].token = equation[i1-1].token;
+							if (equation[i2-1].level == cur_level
+							    && equation[i2+1].level == cur_level
+							    && equation[i2+1].kind == CONSTANT
+							    && equation[i2+1].token.constant == 2.0) {
+								equation[i2].token.operatr = TIMES;
+								equation[i2+1].kind = equation[i2-1].kind;
+								equation[i2+1].token = equation[i2-1].token;
 							} else {
 								if (java_flag) {
-									fprintf(gfp, "Math.pow");
+									cp = "Math.pow";
 								} else {
-									fprintf(gfp, "pow");
+									cp = "pow";
 								}
+								len += fprintf(gfp, cp);
 							}
 							break;
 						case FACTORIAL:
 							if (java_flag) {
-								fprintf(gfp, "MathSupport.fact");
+								cp = "MathSupport.fact";
 							} else {
-								fprintf(gfp, "fact");
+								cp = "fact";
 							}
+							len += fprintf(gfp, cp);
 							break;
 						}
 						break;
 					}
 				}
 				fprintf(gfp, "(");
+				len++;
 			}
 		}
 		switch (equation[i].kind) {
 		case CONSTANT:
 			if (int_flag) {
-				fprintf(gfp, "%.0f", equation[i].token.constant);
+				len += fprintf(gfp, "%.0f", equation[i].token.constant);
 			} else {
 				snprintf(buf, sizeof(buf), "%#.16g", equation[i].token.constant);
 				j = strlen(buf) - 1;
@@ -612,11 +642,11 @@ int		int_flag;	/* integer arithmetic flag */
 					else
 						break;
 				}
-				fprintf(gfp, "%s", buf);
+				len += fprintf(gfp, "%s", buf);
 			}
 			break;
 		case VARIABLE:
-			list_var(equation[i].token.variable, (java_flag ? 2 : 1));
+			len += list_var(equation[i].token.variable, (java_flag ? 2 : 1));
 			fprintf(gfp, "%s", var_str);
 			break;
 		case OPERATOR:
@@ -645,15 +675,17 @@ int		int_flag;	/* integer arithmetic flag */
 				i++;
 				break;
 			}
-			fprintf(gfp, "%s", cp);
+			len += fprintf(gfp, "%s", cp);
 			break;
 		}
 	}
 	j = cur_level - min1;
 	for (; j > 0;) {
 		fprintf(gfp, ")");
+		len++;
 		j--;
 	}
+	return len;
 }
 
 int	cur_line;
@@ -661,6 +693,7 @@ int	cur_pos;
 
 /*
  * Display an equation in fraction format.
+ * Use color if available.
  *
  * Return true if successful.
  */
@@ -677,6 +710,8 @@ int	n;	/* equation number */
 	int	max_line, min_line;
 	int	max2_line, min2_line;
 
+	if (n_lhs[n] <= 0)
+		return false;
 	len = snprintf(buf, sizeof(buf), "#%d: ", n + 1);
 	cur_line = 0;
 	cur_pos = 0;
@@ -709,8 +744,7 @@ make_smaller:
 		len3 = 0;
 	}
 	if (screen_columns && max(len + len2, len3) >= screen_columns) {
-		list1_sub(n, false);
-		return false;
+		return list1_sub(n, false);
 	}
 	fprintf(gfp, "\n");
 	for (i = max_line; i >= min_line; i--) {
@@ -745,12 +779,13 @@ make_smaller:
 
 /*
  * Display a line of an expression if "out_flag" is true.
+ * Use color if available.
  *
  * Return the width of the expression (that is, the required number of screen columns).
  */
 int
 flist(equation, n, out_flag, pos, highp, lowp)
-token_type	*equation;	/* expression */
+token_type	*equation;	/* expression pointer */
 int		n;		/* length of expression */
 int		out_flag;
 int		pos;
@@ -775,7 +810,7 @@ int		pos;
 int		cur_level;
 int		*highp, *lowp;
 {
-	int	i,j,k,l;
+	int	i, j, k, i1;
 	int	l1, l2;
 	int	ii;
 	int	stop_at;
@@ -837,7 +872,7 @@ check_again:
 			j = cur_level - equation[i].level;
 			k = abs(j);
 		}
-		for (l = 1; l <= k; l++) {
+		for (i1 = 1; i1 <= k; i1++) {
 			if (j > 0) {
 				cur_level--;
 				len++;
