@@ -72,6 +72,9 @@ char	**argv;
 	struct winsize	ws;
 #endif
 
+#if	UNIX
+	prog_name = strdup(basename(argv[0]));
+#endif
 #if	CYGWIN
 	dir_path = strdup(dirname1(argv[0]));
 #endif
@@ -126,12 +129,11 @@ char	**argv;
 		printf(_("Copyright (C) 1987-2005 George Gesslein II.\n"));
 	}
 	if (!init_mem()) {
-		fprintf(stderr, _("Not enough memory.\n"));
+		fprintf(stderr, _("%s: Not enough memory.\n"), prog_name);
 		exit(1);
 	}
 	if (!test_mode) {
-		printf(_("%d equation spaces allocated (total size is %ld KBytes).\n"),
-		    n_equations, ((long) n_tokens * sizeof(token_type) * 2L * (n_equations + 2)) / 1000L);
+		printf(_("%d equation spaces available.\n"), N_EQUATIONS);
 	}
 #if	!SECURE
 	if (!test_mode && !load_rc()) {
@@ -154,7 +156,7 @@ char	**argv;
 			printf(_("Operation abruptly aborted.\n"));
 			break;
 		case 14:
-			printf(_("Expression too big!\n"));
+			error(_("Expression too big!"));
 		default:
 			printf(_("Operation aborted.\n"));
 		}
@@ -192,7 +194,7 @@ char	**argv;
 static void
 usage()
 {
-	fprintf(stderr, "Usage: mathomatic [ options ] [ input_files ]\n");
+	fprintf(stderr, "Usage: %s [ options ] [ input_files ]\n", prog_name);
 	fprintf(stderr, "Options:\n");
 	fprintf(stderr, "  -c               Toggle color mode.\n");
 	fprintf(stderr, "  -h               Enable HTML mode.\n");
@@ -205,7 +207,7 @@ void
 fphandler(sig)
 int	sig;
 {
-	printf(_("Floating point exception!\n"));
+	error(_("Floating point exception."));
 	signal(SIGFPE, fphandler);
 	longjmp(jmp_save, 2);
 }
