@@ -25,6 +25,8 @@ char		*cp;		/* string to parse */
 	int		abs_count;
 	int		abs_array[10];
 
+	if (cp == NULL)
+		return(NULL);
 	cp_start = cp;
 	n = 0;
 	*np = 0;
@@ -289,8 +291,9 @@ syntax_error:
 }
 
 /*
- * Parse variable pointed to by "cp".
+ * Parse variable name pointed to by "cp".
  * Variable name is converted to Mathomatic format and stored in "*vp".
+ *
  * Return new string position, or NULL on failure.
  */
 char	*
@@ -299,11 +302,14 @@ long	*vp;
 char	*cp;
 {
 	int	i, j;
-	long	l;
+	long	vtmp;
 	char	buf[MAX_VAR_LEN+1];
 	char	*cp1;
 	int	len;
 
+	if (vp == NULL || cp == NULL) {
+		return(NULL);
+	}
 	if (!isvarchar(*cp)) {
 		return(NULL);
 	}
@@ -317,17 +323,15 @@ char	*cp;
 		}
 		buf[i++] = *cp1++;
 	}
-	if (i == 0)
-		return(NULL);
 	buf[i] = '\0';
 	if (strcasecmp(buf, "sign") == 0) {
-		l = SIGN;
+		vtmp = SIGN;
 		cp += 4;
 	} else if (strcasecmp(buf, "integer") == 0) {
-		l = V_INTEGER;
+		vtmp = V_INTEGER;
 		cp += 7;
 	} else if (strcasecmp(buf, "temp") == 0) {
-		l = V_TEMP;
+		vtmp = V_TEMP;
 		cp += 4;
 	} else {
 		if (strncasecmp(cp, "i#", 2) == 0) {
@@ -351,24 +355,22 @@ char	*cp;
 			}
 			buf[i++] = *cp++;
 		}
-		if (i == 0)
-			return(NULL);
 		buf[i] = '\0';
-		l = 0;
+		vtmp = 0;
 		for (i = 0; var_names[i]; i++) {
 			if (case_sensitive_flag) {
 				if (strcmp(buf, var_names[i]) == 0) {
-					l = i + VAR_OFFSET;
+					vtmp = i + VAR_OFFSET;
 					break;
 				}
 			} else {
 				if (strcasecmp(buf, var_names[i]) == 0) {
-					l = i + VAR_OFFSET;
+					vtmp = i + VAR_OFFSET;
 					break;
 				}
 			}
 		}
-		if (l == 0) {
+		if (vtmp == 0) {
 			if (i >= (MAX_VAR_NAMES - 1)) {
 #if	!SILENT
 				printf(_("Maximum number of variable names reached.\n"));
@@ -385,30 +387,28 @@ char	*cp;
 				return(NULL);
 			}
 			blt(var_names[i], buf, len);
-			l = i + VAR_OFFSET;
+			vtmp = i + VAR_OFFSET;
 			var_names[i+1] = NULL;
 		}
 	}
 	if (isascii(*cp) && isdigit(*cp)) {
-		j = atoi(cp);
+		j = strtol(cp, &cp, 10);
 		if (j < 0 || j > MAX_SUBSCRIPT) {
 			return(NULL);
 		}
-		if (l == SIGN) {
+		if (vtmp == SIGN) {
 			sign_array[j+1] = true;
 		}
-		l += ((long) (j + 1)) << VAR_SHIFT;
-		while (*cp && isascii(*cp) && isdigit(*cp))
-			cp++;
+		vtmp += ((long) (j + 1)) << VAR_SHIFT;
 	}
 	while (*cp == '\'') {
 		cp++;
-		l += PRIME_INCREMENT;
-		if (l < 0) {
+		vtmp += PRIME_INCREMENT;
+		if (vtmp < 0) {
 			return(NULL);
 		}
 	}
-	*vp = l;
+	*vp = vtmp;
 	return cp;
 }
 
