@@ -15,6 +15,7 @@
 #define	true	1
 #define	false	0
 
+void factor_one(double start);
 int positive_integer(double arg);
 void try_factor(double arg);
 void usage(void);
@@ -39,8 +40,7 @@ char	*prog_name;
 int
 main(int argc, char *argv[])
 {
-	int		i, j;
-	double		ii;
+	int		i;
 	double		start;
 	double		stop_at;
 
@@ -89,25 +89,8 @@ askformore:
 		goto endloop;
 	}
 	for (; start <= stop_at; start++) {
-		nn = start;
-		printf("%.0f = ", nn);
-		uno = 0;
-		vv = 1.0 + sqrt(nn);
-		try_factor(2.0);
-		try_factor(3.0);
-		try_factor(5.0);
-		try_factor(7.0);
-		ii = 1.0;
-		while (ii <= vv) {
-			for (j = 0; j < 48; j++) {
-				ii += sq[j];
-				try_factor(ii);
-			}
-		}
-		if (nn > 1.0) {
-			try_factor(nn);
-		}
-		assert(nn == 1.0);
+		printf("%.0f = ", start);
+		factor_one(start);
 		for (i = 0; i < uno; i++) {
 			if (i > 0) {
 				printf(" * ");
@@ -125,6 +108,35 @@ endloop:
 	exit(0);
 }
 
+/*
+ * Factor "start" into the unique array.
+ */
+void
+factor_one(double start)
+{
+	int		j;
+	double		ii;
+
+	nn = start;
+	uno = 0;
+	vv = 1.0 + sqrt(nn);
+	try_factor(2.0);
+	try_factor(3.0);
+	try_factor(5.0);
+	try_factor(7.0);
+	ii = 1.0;
+	while (ii <= vv) {
+		for (j = 0; j < 48; j++) {
+			ii += sq[j];
+			try_factor(ii);
+		}
+	}
+	if (nn > 1.0) {
+		try_factor(nn);
+	}
+	assert(nn == 1.0);
+}
+
 /*
  * Return true if passed double is a positive integer.
  */
@@ -141,8 +153,7 @@ positive_integer(double arg)
 void
 try_factor(double arg)
 {
-retry:
-	if (fmod(nn, arg) == 0.0) {
+	while (fmod(nn, arg) == 0.0) {
 		if (uno > 0 && unique[uno-1] == arg) {
 			ucnt[uno-1]++;
 		} else {
@@ -152,7 +163,6 @@ retry:
 		}
 		nn = nn/arg;
 		vv = 1.0 + sqrt(nn);
-		goto retry;
 	}
 }
 
