@@ -2,15 +2,15 @@
  * Global function prototypes for Mathomatic.
  */
 
-int		clear(), quit(), list_cmd(), simplify(), help(), eliminate();
-int		unfactor(), compare_cmd(), extrema_cmd();
+int		clear(), quit(), list_cmd(), simplify_cmd(), help_cmd(), eliminate_cmd();
+int		unfactor_cmd(), compare_cmd(), extrema_cmd();
 int		read_cmd(), flist_cmd(), calculate();
-int		factor(), derivative(), replace();
-int		save(), print_cmd(), taylor(), limit_cmd();
+int		factor_cmd(), derivative_cmd(), replace_cmd();
+int		save_cmd(), print_cmd(), taylor_cmd(), limit_cmd();
 int		copy_cmd(), divide_cmd(), pause_cmd();
-int		edit(), real(), imaginary(), tally();
+int		edit_cmd(), real_cmd(), imaginary_cmd(), tally_cmd();
 int		roots_cmd(), set_cmd(), code_cmd(), optimize_cmd(), push_cmd();
-int		sum_cmd(), product_cmd(), integrate_cmd(), nintegrate(), laplace_cmd();
+int		sum_cmd(), product_cmd(), integrate_cmd(), nintegrate_cmd(), laplace_cmd();
 char		*skip_space(), *skip_param();
 char		*skip_num();
 char		*getstring();
