@@ -151,8 +151,8 @@ int		poly_flag, quick_flag;
 		do {
 			do {
 				simp_ssub(equation, np, 0L, 1.0, false, true, 1);
-			} while (uf_power(equation, np));
-		} while (polydiv_simp(equation, np));
+			} while (polydiv_simp(equation, np));
+		} while (uf_power(equation, np));
 	} while (super_factor(equation, np, 2));
 	side_debug(2, equation, *np);
 	unsimp_power(equation, np);
@@ -762,10 +762,6 @@ int		*np, i;
 	if (equation[i+1].level != level + 1 || equation[i+1].kind != CONSTANT
 	    || equation[i+1].token.constant == 1.0)
 		return false;
-#if	false
-	if (equation[i-1].token.constant < 0.0 && fmod(equation[i+1].token.constant, 1.0) != 0.0)
-		return false;	/* don't make imaginary numbers */
-#endif
 	j = i + 2;
 	if (j >= *np || equation[j].level != level + 1)
 		return false;
@@ -1107,15 +1103,13 @@ double	k2;
 		}
 		if (*k1p == 0.0 && k2 < 0.0) {
 #if	!SILENT
-			printf(_("Divide by zero error!\n"));
+			printf(_("Divide by zero error.\n"));
 #endif
 			domain_check = false;
 			longjmp(jmp_save, 2);
 		}
 		if (*k1p == 0.0 && k2 == 0.0) {
-#if	!SILENT
-			printf(_("Warning: 0 raised to the power of 0.\n"));
-#endif
+			debug_string(0, _("Warning: 0 raised to the power of 0."));
 			d = 1.0;	/* some people don't know 0^0 */
 		} else {
 			d = pow(*k1p, k2);
@@ -1144,13 +1138,13 @@ double	k2;
 #else
 		if (*k1p < 0.0) {
 #if	!SILENT
-			printf(_("Factorial not defined for negative numbers!\n"));
+			printf(_("Factorial not defined for negative numbers.\n"));
 #endif
 			longjmp(jmp_save, 2);
 		}
 		if (fmod(*k1p, 1.0) != 0.0) {
 #if	!SILENT
-			printf(_("Factorial operand not integer!\n"));
+			printf(_("Factorial operand not integer.\n"));
 #endif
 			longjmp(jmp_save, 2);
 		}
