@@ -208,7 +208,6 @@ int		symbolic_flag;
 	int		loc2, len2;
 	int		loct, lent;
 	int		count;
-	int		gcd_count;
 	jmp_buf		save_save;
 	int		div_flag;
 	int		vc, cnt;
@@ -312,7 +311,7 @@ int		symbolic_flag;
 		tlhs[n_tlhs].token.variable = v;
 		n_tlhs++;
 		uf_simp(tlhs, &n_tlhs);
-		if ((gcd_count = poly_gcd(&equation[loc1], len, tlhs, n_tlhs, v)) == 0)
+		if (poly_gcd(&equation[loc1], len, tlhs, n_tlhs, v) == 0)
 			break;
 		if (!level1_plus(tlhs, n_tlhs))
 			break;
@@ -321,11 +320,7 @@ int		symbolic_flag;
 		len = n_trhs;
 		switch (count) {
 		case 1:
-#if	!SILENT
-			if (debug_level > 0) {
-				printf(_("Polynomial with repeated factor factored.  Difficulty = %d...\n"), gcd_count);
-			}
-#endif
+			debug_string(1, "Polynomial with repeated factor factored.");
 			len_first = n_tlhs;
 			loc2 = loc1;
 			break;
@@ -375,7 +370,7 @@ next_v:
 				printf("\n");
 			}
 #endif
-			if ((gcd_count = poly_gcd(&equation[loc1], len, tlhs, n_tlhs, 0L)) == 0)
+			if (poly_gcd(&equation[loc1], len, tlhs, n_tlhs, 0L) == 0)
 				goto next_v;
 			if (!level1_plus(tlhs, n_tlhs))
 				goto next_v;
@@ -391,10 +386,6 @@ next_v:
 	}
 skip_factor:
 	blt(jmp_save, save_save, sizeof(jmp_save));
-	if (modified || single_modified) {
-		debug_string(3, "Derived factors are:");
-		side_debug(3, equation, *np);
-	}
 	if (modified) {
 /* Repeated factor was factored out. */
 /* See if we can factor out more of the repeated factor. */
@@ -1255,7 +1246,7 @@ long		*vp;		/* variable pointer to base variable */
 	jmp_buf		save_save;
 
 	old_partial = partial_flag;
-	partial_flag = false;
+	partial_flag = false;	/* We want full unfactoring. */
 	blt(save_save, jmp_save, sizeof(jmp_save));
 	if ((i = setjmp(jmp_save)) != 0) {
 		blt(jmp_save, save_save, sizeof(jmp_save));
@@ -1271,6 +1262,9 @@ long		*vp;		/* variable pointer to base variable */
 	return j;
 }
 
+/*
+ * Do the actual polynomial division.
+ */
 static int
 poly_div_sub(d1, len1, d2, len2, vp)
 token_type	*d1;
@@ -1284,14 +1278,13 @@ long		*vp;
 	int		t2, len_t2;
 	int		sign;
 	int		divide_flag;
-	double		last_power;
-	double		divisor_power;
+	double		last_power, divisor_power, d;
 	int		sum_size;
-	double		d;
-	int		last_count, count;
+	int		last_count, count;	/* dividend number of terms raised to the highest power */
 
 	if (len1 > n_tokens || len2 > n_tokens)
 		return false;
+	/* Copy the source polynomials to where we want them (tlhs and trhs). */
 	if (trhs != d1) {
 		blt(trhs, d1, len1 * sizeof(token_type));
 		n_trhs = len1;
@@ -1300,9 +1293,11 @@ long		*vp;
 		blt(tlhs, d2, len2 * sizeof(token_type));
 		n_tlhs = len2;
 	}
+	/* Do the basic unfactoring and simplification. */
 	uf_simp(trhs, &n_trhs);
 	uf_simp(tlhs, &n_tlhs);
 	if (*vp == 0) {
+		/* Select the best base variable. */
 		if (find_highest_count(trhs, n_trhs, tlhs, n_tlhs, vp) <= 0)
 			return false;
 	}
@@ -1314,6 +1309,7 @@ long		*vp;
 		side_debug(3, tlhs, n_tlhs);
 	}
 #endif
+	/* Determine divide_flag and if the polynomials can be divided. */
 	divide_flag = 2;
 	last_count = find_greatest_power(trhs, n_trhs, vp, &last_power, &t1, &len_t1, &divide_flag);
 	find_greatest_power(tlhs, n_tlhs, vp, &divisor_power, &t2, &len_t2, &divide_flag);
@@ -1325,6 +1321,7 @@ long		*vp;
 			return false;
 		}
 	}
+	/* Initialize the quotient. */
 	n_quotient = 1;
 	quotient[0].level = 1;
 	quotient[0].kind = CONSTANT;
@@ -1334,6 +1331,7 @@ long		*vp;
 	blt(divisor, tlhs, n_tlhs * sizeof(token_type));
 	n_divisor = n_tlhs;
 	sum_size = n_trhs + n_quotient;
+	/* Loop until polynomial division is finished. */
 	for (;;) {
 		sign = PLUS;
 		if (t1 > 0 && trhs[t1-1].token.operatr == MINUS)
@@ -1413,6 +1411,7 @@ long		*vp;
 		side_debug(4, trhs, n_trhs);
 		count = find_greatest_power(trhs, n_trhs, vp, &d, &t1, &len_t1, &divide_flag);
 		if (d < divisor_power) {
+			/* Success!  Polynomial division ends here. */
 			blt(tlhs, quotient, n_quotient * sizeof(token_type));
 			n_tlhs = n_quotient;
 			side_debug(3, tlhs, n_tlhs);
@@ -1441,8 +1440,6 @@ long		*vp;
 	}
 }
 
-#define	MAX_TRIES	100
-
 /*
  * Do smart division.
  *
@@ -1465,6 +1462,8 @@ int		len1;		/* length of dividend */
 token_type	*d2;		/* pointer to divisor */
 int		len2;		/* length of divisor */
 {
+#define	MAX_TRIES	100
+
 	int		i, j, k;
 	int		t1, len_t1;
 	int		t2, len_t2;
@@ -1485,11 +1484,13 @@ int		len2;		/* length of divisor */
 	n_trhs = len1;
 	blt(tlhs, d2, len2 * sizeof(token_type));
 	n_tlhs = len2;
+	/* Do the basic unfactoring and simplification of the dividend and divisor. */
 	uf_simp_no_repeat(trhs, &n_trhs);
 	uf_simp_no_repeat(tlhs, &n_tlhs);
 	debug_string(3, "smart_div() starts:");
 	side_debug(3, trhs, n_trhs);
 	side_debug(3, tlhs, n_tlhs);
+	/* Find which divisor term to use. */
 	dcount = 0;
 	for (i = 0, j = 0, k = 0, len_t2 = 0, vflag = false;; i++) {
 		if (i >= n_tlhs || (tlhs[i].kind == OPERATOR && tlhs[i].level == 1
@@ -1513,6 +1514,7 @@ int		len2;		/* length of divisor */
 	}
 	if (len_t2 <= 0)
 		return false;
+	/* Initialize the quotient. */
 	n_quotient = 1;
 	quotient[0].level = 1;
 	quotient[0].kind = CONSTANT;
@@ -1568,7 +1570,7 @@ try_one:
 					dcount = 1;
 					t2 = 0;
 					len_t2 = n_divisor;
-					goto try_one;
+					goto try_one;	/* Try using the whole divisor as the divisor term. */
 				}
 				return false;
 			}
@@ -1584,7 +1586,7 @@ end_div2:
 			n_tlhs = n_quotient;
 			side_debug(3, tlhs, n_tlhs);
 			side_debug(3, trhs, n_trhs);
-			return true;
+			return true;	/* Success! */
 		}
 		t1 = term_pos;
 		len_t1 = term_size;
